/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.net.URL;
import java.util.Properties;

import javax.help.HelpSet;

import org.cytoscape.application.CyApplicationManager;
import org.cytoscape.application.swing.CyAction;
import org.cytoscape.application.swing.CyHelpBroker;
import org.cytoscape.application.swing.CySwingApplication;
import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.model.CyNetworkFactory;
import org.cytoscape.model.CyNetworkManager;
import org.cytoscape.model.CyTableFactory;
import org.cytoscape.model.subnetwork.CyRootNetworkManager;
import org.cytoscape.service.util.AbstractCyActivator;
import org.cytoscape.view.layout.CyLayoutAlgorithmManager;
import org.cytoscape.view.model.CyNetworkViewFactory;
import org.cytoscape.view.model.CyNetworkViewManager;
import org.cytoscape.view.model.events.NetworkViewAboutToBeDestroyedListener;
import org.cytoscape.view.model.events.NetworkViewAddedListener;
import org.cytoscape.work.swing.DialogTaskManager;
import org.osgi.framework.BundleContext;


public class CyActivator extends AbstractCyActivator {

	/*******************
	 *    ATTRIBUTES   *
	 *******************/

	// Cytoscape factories
	protected static CyNetworkFactory networkFactory;
	protected static CyTableFactory cyDataTableFactoryServiceRef;
	protected static CyNetworkViewFactory networkViewFactory;

	// Cytoscape managers
	protected static CyApplicationManager cyApplicationManagerService;
	protected static CyNetworkManager cyNetworkManagerServiceRef;
	protected static CyRootNetworkManager cyRootNetworkFactoryServiceRef;
	protected static CyNetworkViewManager networkViewManager;
	protected static CyLayoutAlgorithmManager layoutAlgoManager;

	// Tasks executor
	protected static DialogTaskManager taskManager;

	// Help pages broker
	protected static CyHelpBroker helpBroker;

	// Reference communication object
	protected static JEPETTOCommunication communication;

	// Reference results object
	protected static JEPETTOResultsPanel enrichmentResultsTab;
	protected static JEPETTOResultsPanel topologyResultsTab;

	// Number of analysis counter
	protected static int overallEnrichmentAnalysisNumber = 0;
	protected static int overallTopologyAnalysisNumber = 0;

	// Default expansion parameters
	protected static double associationValue = 1.0;
	protected static double coverageValue = 0.3;
	protected static double triangleValue = 0.1;



	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 */
	public CyActivator() {
		super();
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Start method to set up the plugin
	 */
	@Override
	public void start(BundleContext context) {
		// Get cytoscape swing services
		CySwingApplication cytoscapeDesktopService = getService(context, CySwingApplication.class);

		// Initialize communication object
		communication = new JEPETTOCommunication();

		// Initialize cytoscape control panel object
		JEPETTOServerPanel myCytoPanel = new JEPETTOServerPanel();

		// Initialize cytoscape results object
		enrichmentResultsTab = new JEPETTOResultsPanel(cytoscapeDesktopService, " Enrichment ");
		enrichmentResultsTab.setToolTipText("Results for JEPETTO enrichment analysis");
		topologyResultsTab = new JEPETTOResultsPanel(cytoscapeDesktopService, " Topology ");
		topologyResultsTab.setToolTipText("Results for JEPETTO topological analysis");

		// Set up menu items
		JEPETTOMenuActionPlugin menuEnrichmentAnalysis = new JEPETTOMenuActionPlugin(cytoscapeDesktopService, myCytoPanel, "Enrichment analysis");
		JEPETTOMenuActionPlugin menuTopologicalAnalysis = new JEPETTOMenuActionPlugin(cytoscapeDesktopService, myCytoPanel, "Topological analysis");
		JEPETTOMenuActionPlugin menuTutorial = new JEPETTOMenuActionPlugin(cytoscapeDesktopService, myCytoPanel, "Online tutorial");
		JEPETTOMenuActionPlugin menuHelp = new JEPETTOMenuActionPlugin(cytoscapeDesktopService, myCytoPanel, "Help");
		JEPETTOMenuActionPlugin menuAbout = new JEPETTOMenuActionPlugin(cytoscapeDesktopService, myCytoPanel, "About");

		// Handle network views creation and deletion events
		JEPETTONetworkViewListener viewEventListener = new JEPETTONetworkViewListener();

		// register JEPETTO panel
		registerService(context, myCytoPanel, CytoPanelComponent.class, new Properties());

		// register result tabs
		registerService(context, enrichmentResultsTab, CytoPanelComponent.class, new Properties());
		registerService(context, topologyResultsTab, CytoPanelComponent.class, new Properties());

		// register menu actions
		registerService(context, menuAbout, CyAction.class, new Properties());
		registerService(context, menuHelp, CyAction.class, new Properties());
		registerService(context, menuTutorial, CyAction.class, new Properties());
		registerService(context, menuEnrichmentAnalysis, CyAction.class, new Properties());
		registerService(context, menuTopologicalAnalysis, CyAction.class, new Properties());

		// register network view listener
		registerService(context, viewEventListener, NetworkViewAddedListener.class, new Properties());
		registerService(context, viewEventListener, NetworkViewAboutToBeDestroyedListener .class, new Properties());

		// Get cytoscape services
		cyApplicationManagerService = getService(context, CyApplicationManager.class);
		cyNetworkManagerServiceRef = getService(context,CyNetworkManager.class);
		cyDataTableFactoryServiceRef = getService(context,CyTableFactory.class);
		cyRootNetworkFactoryServiceRef = getService(context,CyRootNetworkManager.class);
		networkFactory = getService(context, CyNetworkFactory.class);
		networkViewFactory = getService(context, CyNetworkViewFactory.class);
		networkViewManager = getService(context, CyNetworkViewManager.class);
		layoutAlgoManager = getService(context, CyLayoutAlgorithmManager.class);
		taskManager = getService(context, DialogTaskManager.class);
		helpBroker = getService(context, CyHelpBroker.class);

		// add plugin help set to the Cytoscape help
		String HELP_SET_NAME = "/help/jhelpset";
		ClassLoader classLoader = getClass().getClassLoader();
		HelpSet cytoscapeHelp = helpBroker.getHelpSet();

		try {
			URL helpSetURL = HelpSet.findHelpSet(classLoader, HELP_SET_NAME);
			HelpSet newHelpSet = new HelpSet(classLoader, helpSetURL);
			cytoscapeHelp.add(newHelpSet);
		} catch (Exception e) {
			System.err.println("Could not find help set: " + HELP_SET_NAME);
		}
	}
}
