/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import javax.imageio.ImageIO;

import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpResponseException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.client.LaxRedirectStrategy;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JEPETTOCommunication {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	private final Logger logger = LoggerFactory.getLogger(getClass());


	/*******************
	 *	 METHODS	 *
	 *******************/

	/**
	 * Handles the HTTP response and does connection management.
	 * @param response
	 * @return response as String
	 *
	 */
	private final ResponseHandler<String> responseHandler = new ResponseHandler<String>() {
		@Override
		public String handleResponse(final HttpResponse response)
		throws IOException, HttpResponseException, ClientProtocolException {

			int code = response.getStatusLine().getStatusCode();
			HttpEntity entity = response.getEntity();

			logger.debug("response code:" + Integer.toString(code));

			if (code >= 300) {
				String reason = response.getStatusLine().getReasonPhrase();
				logger.error("HTTP {} response: {}.", code, reason);
				throw new HttpResponseException(code, reason);
			}
			if (entity == null) {
				logger.warn("Empty response.");
				throw new ClientProtocolException("Response contains no content.");
			}
			return EntityUtils.toString(entity);
		}
	};

	/**
	 * Upload an interaction network as post parameter
	 * @param url
	 * @param file
	 * @return uploaded file html page
	 */
	protected String uploadFile(String url, File file)
	throws ExperimentException {

		FileBody network = new FileBody(file);
		StringBody param1 = new StringBody("yes", ContentType.TEXT_PLAIN);
		StringBody param2 = new StringBody("Upload", ContentType.TEXT_PLAIN);

		HttpEntity multipartEntity = MultipartEntityBuilder.create()
			.addPart("file", network)
			.addPart("upFile", param1)
			.addPart("submit", param2)
			.build();

		HttpPost post = new HttpPost(url);
		post.setEntity(multipartEntity);

		CloseableHttpClient client = HttpClients.createDefault();
		try {
			return client.execute(post, responseHandler);
		} catch (ConnectTimeoutException e) {
			throw new ExperimentException("Connection timeout", e);
		} catch (HttpResponseException e) {
			throw new ExperimentException("Server error", e);
		} catch (IOException e) {
			throw new ExperimentException("I/O error", e);
		} finally {
			try {
				client.close();
			} catch (IOException e) {
				logger.warn("HttpClient connection not closed. {}",  e.getMessage());
			}
		}
	}

	/**
	 * Send parameters to server
	 * @param url
	 * @param parameters
	 * @return the resulting html page
	 */
	protected String sendForm(String url, List<NameValuePair> parameters)
	throws ExperimentException {

		// force http client to follow a redirect after post
		LaxRedirectStrategy lrs = new LaxRedirectStrategy();
		CloseableHttpClient client = HttpClients.custom().setRedirectStrategy(lrs).build();

		HttpPost post = new HttpPost(url);
		post.setEntity(new UrlEncodedFormEntity(parameters, Consts.UTF_8));

		try {
			return client.execute(post, responseHandler);
		} catch (ConnectTimeoutException e) {
			throw new ExperimentException("Connection timeout", e);
		} catch (HttpResponseException e) {
			throw new ExperimentException("Server error", e);
		} catch (IOException e) {
			throw new ExperimentException("I/O error", e);
		} finally {
			try {
				client.close();
			} catch (IOException e) {
				logger.warn("HttpClient connection not closed. {}",  e.getMessage());
			}
		}
	}

	/**
	 * Get the regression plot
	 * @param url
	 * @param file
	 * @return the regression plot
	 */
	protected Image retrieveImage(String url)
	throws MalformedURLException, IOException {

		URL location = new URL(url);
		Image image = ImageIO.read(location);
		return image;
	}

	/**
	 * Connect to one of the generated files during an analysis
	 * @param url
	 * @param file
	 * @return the output file as string
	 */
	protected String readFile(String url)
	throws ExperimentException {

		String page;
		CloseableHttpClient client = HttpClients.createDefault();

		try {
			HttpGet get = new HttpGet(url);
			page = client.execute(get, responseHandler);
		} catch (ConnectTimeoutException e) {
			throw new ExperimentException("Connection timeout", e);
		} catch (HttpResponseException e) {
			throw new ExperimentException("Server error", e);
		} catch (IOException e) {
			throw new ExperimentException("I/O error", e);
		} finally {
			try {
				client.close();
			} catch (IOException e) {
				logger.warn("HttpClient connection not closed. {}",  e.getMessage());
			}
		}
		return page;
	}
}
