/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.lang.Math;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableColumnModel;

import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskIterator;
import org.cytoscape.work.TaskMonitor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Custom exception class. Thrown when some server issue or
 * a connection problem occurs.
 */
class ExperimentException extends Exception {
	private static final long serialVersionUID = -159298599523429031L;

	public ExperimentException(String message) {
		super("Unexpected server error. " + message);
	}

	public ExperimentException(String message, Exception e) {
		super("Server connection failed. Try to run the experiment again.\n"
			+ message + " (" + e.getMessage() + ").");
	}
}


public class JEPETTOEnrichNetMethods extends JEPETTOGenericServerMethods {

	/*************
	 * CONSTANTS *
	 * ***********/

	private static final String URL = "http://www.enrichnet.org/";

	private static final String[] PARAMETERS = {"SESSION_NAME", "pathway", "example", "network", "identifier", "geneset"};

	private static final String[] DEFAULTS = {null, "KEGG///kegg+BioCarta///biocarta+GO Biological Process///gobp+Wiki Pathways///wiki+Reactome///reactome+NCI Pathway Interaction///nci+InterPro///interpro+GO Molecular Function///gomf+GO Cellular Component///gocc+CORUM complexes///corum", null, "Default (STRING)///string+Sparse (STRING 900)///string900+Sparse (Bossi & Lehner)///ppi+Integrative physical network (PSICQUIC)///hppi2012", "ENSEMBL ID///ensembl_gene_id+HGNC SYMBOL///hgnc_symbol+ENTREZ GENE///entrezgene+UNIPROT/SWISSPROT ID///uniprot_swissprot+UNIPROT/SWISSPROT ACC///swissprot_acc+REFSEQ DNA///refseq_dna+PROTEIN ID///protein_id+Affymetrix HC G110///affy_hc_g110+Affymetrix HG Focus///affy_hg_focus+Affymetrix HG U133 plus 2///affy_hg_u133_plus_2+Affymetrix HG U133a///affy_hg_u133a+Affymetrix HG U133a 2///affy_hg_u133a_2+Affymetrix HG U133b///affy_hg_u133b+Affymetrix HG U95a///affy_hg_u95a+Affymetrix HG U95av2///affy_hg_u95av2+Affymetrix HG U95b///affy_hg_u95b+Affymetrix HG U95c///affy_hg_u95c+Affymetrix HG U95d///affy_hg_u95d+Affymetrix HG U95e///affy_hg_u95e+Affymetrix HuEx-1_0-st-v2///affy_huex_1_0_st_v2+Affymetrix HuGene-1_0-st-v1///affy_hugene_1_0_st_v1+Affymetrix HuGene FL///affy_hugenefl+Affymetrix U133 X3P///affy_u133_x3p+Agilent CGH 44b///agilent_cgh_44b+Agilent Wholegenome///agilent_wholegenome+Illumina HumanWG-6 v1///illumina_humanwg_6_v1+Illumina HumanWG-6 v2///illumina_humanwg_6_v2+Illumina HumanWG-6 v3///illumina_humanwg_6_v3", null};

	private static final Pattern ID_PATTERN = Pattern.compile("filecreated.php\\?temp=([a-z0-9]{5}_[0-9]{10})");

	/*******************
	 *    ATTRIBUTES   *
	 *******************/

	private static final Logger logger = LoggerFactory.getLogger(JEPETTOEnrichNetMethods.class);

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = -6826326928872005950L;

	// EnrichNet experiment ID
	protected String idEnrichNet;

	// Network visualization variables
	private HashMap<String, String> pathwaysList = new HashMap<String, String>();
	private JTable rankingTable;
	private HashMap<String, Integer> networkComputationIndexesMap;

	/*protected HashMap<String, String> conversionEnsemblId = new HashMap<String, String>();
	protected final ArrayList<String[]> edges = new ArrayList<String[]>();
	protected final HashMap<String, String> category = new HashMap<String, String>();*/

	// Constructor parameters
	private final String molecularNetwork;
	private final String idFormat;
	private final boolean isTissueSpecific;
	private final String geneSet;
	private final String annotationDb;
	protected static int statusTask = 0;

	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param component
	 * @param molecularNetwork
	 * @param idFormat
	 * @param isTissueSpecific
	 * @param geneSet
	 * @param annotationDb
	 */
	public JEPETTOEnrichNetMethods(String molecularNetwork, String idFormat,
		boolean isTissueSpecific, String geneSet, String annotationDb) {

		this.molecularNetwork = molecularNetwork;
		this.idFormat = idFormat;
		this.isTissueSpecific = isTissueSpecific;
		this.geneSet = geneSet;
		this.annotationDb = annotationDb;

		analysis(molecularNetwork, idFormat, isTissueSpecific, geneSet, annotationDb);
	}

	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Start an experiment with EnrichNet
	 * @param molecularNetwork
	 * @param idFormat
	 * @param isTissueSpecific
	 * @param geneSet
	 * @param annotationDb
	 * @throws java.lang.InterruptedException;
	 */
	private void analysis(final String molecularNetwork, final String idFormat,
		final boolean isTissueSpecific, final String geneSet,
		final String annotationDb) {

		AbstractTask analysisTask = new AbstractTask () {

			@Override
			public void run(TaskMonitor taskMonitor) throws ExperimentException {

				taskMonitor.setTitle("Enrichment analysis");
				taskMonitor.setProgress(-1);
				taskMonitor.setStatusMessage("Sending request");

				boolean isLongExperiment = false;

				String experimentSettingsTooltip = "<html>Molecular network: " + molecularNetwork + "<br>" +
						"Identifier format: " + idFormat + "<br>" +
						"Reference database: " + annotationDb + "<br>" +
						"Tissue-specificity: "; // tooltip to keep trace of enrichment experiments

				// start EnrichNet session
				String htmlSessionPage = null;

				if ((molecularNetwork != "Default (STRING)") &
						(molecularNetwork != "Sparse (STRING 900)") &
						(molecularNetwork != "Sparse (Bossi & Lehner)") &
						(molecularNetwork != "Integrative physical network (PSICQUIC)")) {

					isLongExperiment = true;

					// upload user network
					taskMonitor.setTitle("Enrichment analysis [user network: time-consuming]");
					htmlSessionPage = CyActivator.communication.uploadFile(
						URL + " index.php?network=uploaded", new File(molecularNetwork));
				} else {
					// read main page
					htmlSessionPage = CyActivator.communication.readFile(URL+"index.php");
				}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				// get session name
				String sessionName = getID(htmlSessionPage, "value=\"(SESS[a-z_0-9]{13})\"");
				logger.info("Session name: {}", sessionName);

				// set form parameters
				List<String> names1 = Arrays.asList(PARAMETERS);
				List<String> defaults1 = Arrays.asList(DEFAULTS);
				List<String> values1 = Arrays.asList(sessionName, annotationDb,
					"0", "uploaded", idFormat, geneSet);
				ArrayList<String> names = new ArrayList<String>(names1);
				ArrayList<String> defaults = new ArrayList<String>(defaults1);
				ArrayList<String> values = new ArrayList<String>(values1);

				// extra parameter for tissue specific enrichment
				if (isTissueSpecific) {
					names.add("tissuespec");
					defaults.add(null);
					values.add("yes");
					experimentSettingsTooltip += "yes</html>";
				} else {
					experimentSettingsTooltip += "no</html>";
				}

				// for user submitted network don't use default values
				if ((molecularNetwork != "Default (STRING)") &
					(molecularNetwork != "Sparse (STRING 900)") &
					(molecularNetwork != "Sparse (Bossi & Lehner)") &
					(molecularNetwork != "Integrative physical network (PSICQUIC)")) {

					defaults.set(3, null);
				}

				// convert parameters to string
				String parameters = getPostParameters(names.toArray(new String[0]),
					values.toArray(new String[0]), defaults.toArray(new String[0]));

				// submit experiment form
				String html = CyActivator.communication.sendForm(URL+"index.php?tmpdat=result",
					setPostParameters(parameters));

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				if (html == null) {
					throw new ExperimentException("EnrichNet submission page is empty.");
				}

				logger.info("Form response length: {}", html.length());

				// find experiment ID
				Matcher matcher = ID_PATTERN.matcher(html);
				if (matcher.find()) {
					idEnrichNet = matcher.group(1);
					logger.info("Experiment ID: {}", idEnrichNet);
				} else {
					throw new ExperimentException("EnrichNet experiment ID not found.");
				}

				if (isLongExperiment) {
					taskMonitor.setStatusMessage("Waiting for results (may take longer than 10 minutes)");
				} else {
					if (annotationDb.startsWith("GO Bio")) {
						taskMonitor.setStatusMessage("Waiting for results (may take up to 5 minutes)");
					} else if (annotationDb.startsWith("Inter") || annotationDb.startsWith("GO Mol")) {
						taskMonitor.setStatusMessage("Waiting for results (may take up to 2 minutes)");
					} else {
						taskMonitor.setStatusMessage("Waiting for results (may take up to 1 minute)");
					}
				}

				long startTime = System.nanoTime();

				// wait until results are ready
				int waiting = 0;
				while (true) {
					// sleep for 2 seconds
					try {
						Thread.sleep(2000);
					} catch (InterruptedException exception) {}
					waiting += 2;

					if (waiting > 600) {
						throw new ExperimentException("Experiment took more than 10 minutes to complete.");
					}

					String response = CyActivator.communication.readFile(URL+"filecreated.php?temp="+idEnrichNet);
					if (response.equals("Success!")) {
						logger.info("Results file created.");
						break;
					}
					logger.info("Waiting for results for {} seconds.", waiting);

					// check if the task has been cancelled
					if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}
				}

				// finish measuring the experiment processing time
				// and start measuring download time
				long processingTime = System.nanoTime() - startTime;
				startTime = System.nanoTime();

				logger.info("Reading results page.");

				// read results page
				html = CyActivator.communication.readFile(URL+"pages/tmp/"+idEnrichNet+"/result.php");
				if (html == null) {
					throw new ExperimentException("EnrichNet result page is empty.");
				}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				logger.info("Results page length: {}", html.length());

				taskMonitor.setStatusMessage("Downloading results");

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				// Get specific 2D plot values
				Double[] plotValues = get2DPlotValues(html);
				JPanel panel2dPlot = new JPanel();

				// TODO: why this is happening? what does it mean?
				if (plotValues[0] == 0 && plotValues[1] == 0) {
					logger.warn("Regression plot not generated.");
				} else {
					// Retrieve the 2D plot
					logger.info("Reading regression plot file.");
					panel2dPlot = get2DPlot(idEnrichNet);
				}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				// Ranking table
				logger.info("Reading ranking data.");
				prepareRanking(idEnrichNet, plotValues[1], annotationDb);

				TableColumnModel tcm = rankingTable.getColumnModel();
				tcm.getColumn(1).setPreferredWidth(40);
				tcm.getColumn(2).setPreferredWidth(40);
				tcm.getColumn(3).setPreferredWidth(20);

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				JTable tissueSpecTable = null;
				if (isTissueSpecific){
					logger.info("Reading tissue data file.");
					tissueSpecTable = getTissueSpecificities(idEnrichNet);
					tissueSpecTable = getTableFromFile(URL+"pages/tmp/"+idEnrichNet+"/tissue_specificity_matrix.txt", "readDataFromTissueSpecificity", new String[]{"", "atrioventricular node", "trigeminal ganglion", "globus pallidus", "ciliary ganglion", "superior cervical ganglion", "tongue", "testis - interstitial", "testis", "skeletal muscle", "dorsal root ganglion", "lymph node", "bone marrow", "thymus", "trachea", "tonsil", "salivary gland", "appendix", "prefrontal cortex", "amygdala", "subthalamic nucleus", "testis - seminiferous tubule", "pons", "temporal lobe", "skin", "parietal lobe", "caudate nucleus", "liver", "thalamus", "cerebellum", "cingulate cortex", "kidney", "cerebellum peduncles", "occipital lobe", "hypothalamus", "adrenal gland", "medulla oblongata", "heart", "fetal thyroid", "adrenal cortex", "thyroid", "whole blood", "white blood cells", "fetal brain", "fetal liver", "tumor", "placenta", "smooth muscle", "olfactory bulb", "ovary", "lung", "whole brain", "uterus corpus", "uterus", "spinal cord", "prostate", "pancreas", "fetal lung", "adipose tissue", "pituitary", "beta cell islets"}, new String[]{"", "atrioventricular node", "trigeminal ganglion", "globus pallidus", "ciliary ganglion", "superior cervical ganglion", "tongue", "testis - interstitial", "testis", "skeletal muscle", "dorsal root ganglion", "lymph node", "bone marrow", "thymus", "trachea", "tonsil", "salivary gland", "appendix", "prefrontal cortex", "amygdala", "subthalamic nucleus", "testis - seminiferous tubule", "pons", "temporal lobe", "skin", "parietal lobe", "caudate nucleus", "liver", "thalamus", "cerebellum", "cingulate cortex", "kidney", "cerebellum peduncles", "occipital lobe", "hypothalamus", "adrenal gland", "medulla oblongata", "heart", "fetal thyroid", "adrenal cortex", "thyroid", "whole blood", "white blood cells", "fetal brain", "fetal liver", "tumor", "placenta", "smooth muscle", "olfactory bulb", "ovary", "lung", "whole brain", "uterus corpus", "uterus", "spinal cord", "prostate", "pancreas", "fetal lung", "adipose tissue", "pituitary", "beta cell islets"}, false, 0);
				}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				// set delay proportional to processing time (1/20)
				// and discount the download time used already
				long downloadTime = (System.nanoTime() - startTime) / 1000000;
				long delay = Math.max(0, processingTime / 20000000 - downloadTime);

				// wait some extra time for mapping table to appear
				try {
					Thread.sleep(delay);
				} catch (InterruptedException exception) {}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				JTable mappingTable = null;
				logger.info("Reading mapping data file.");
				mappingTable = getTableFromFile(URL+"pages/tmp/"+idEnrichNet+"/maptable.html", "readDataFromMaptable", new String[]{"Initial","Converted","Presence"}, new String[]{"Initial identifier", "Converted identifier", "Present in network"}, false, 4);

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the enrichment analysis."); return;}

				// Increase number of analysis by one
				CyActivator.overallEnrichmentAnalysisNumber++;

				// Update results panel
				if (tissueSpecTable == null) {
					new JEPETTOEnrichNetResultPanel(rankingTable,
							mappingTable,
							panel2dPlot,
							plotValues,
							idEnrichNet,
							annotationDb,
							pathwaysList,
							networkComputationIndexesMap,
							experimentSettingsTooltip);
				} else {
					new JEPETTOEnrichNetResultPanel(rankingTable,
							mappingTable,
							tissueSpecTable,
							panel2dPlot,
							plotValues,
							idEnrichNet,
							annotationDb,
							pathwaysList,
							networkComputationIndexesMap,
							experimentSettingsTooltip);
				}
				CyActivator.enrichmentResultsTab.getPanel();
			}
		};

		TaskIterator it = new TaskIterator(analysisTask);
		CyActivator.taskManager.execute(it);
	}

	/**
	 * Creates a map with each pathway / cellular process
	 * associated to its specific index for graph computations in EnrichNet
	 * @param rankingTable
	 * @return a map containing each pathway and its computation index for enrichnet requests
	 */
	private HashMap<String, Integer> getNetworkComputationsIndexes(JTable rankingTable) {
		HashMap<String, Integer> graphComputationIndexMap = new HashMap<String, Integer>(rankingTable.getRowCount());
		int i = 0;
		while (i < rankingTable.getRowCount()) {
			graphComputationIndexMap.put((String) rankingTable.getModel().getValueAt(i, 0), new Integer(i+1));
			i++;
		}
		return graphComputationIndexMap;
	}

	/**
	 * Compute a cytoscape network properties
	 * @param id
	 * @param pathwayIndex
	 * @param pathwayName
	 * @param enrichnetPathways
	 * @param database
	 */
	protected static void computeGraph(final String id, final int pathwayIndex, final String pathwayName, final HashMap<String, String> enrichnetPathways, final String database) {
		AbstractTask graphTask = new AbstractTask() {

			@Override
			public void run(TaskMonitor taskMonitor) throws Exception {
				taskMonitor.setTitle("Generating network...");
				taskMonitor.setProgress(-1);
				taskMonitor.setStatusMessage("Sending request");

				JEPETTOPathExpandPanel pathExpand = new JEPETTOPathExpandPanel(database);

				taskMonitor.setStatusMessage("Waiting for results");

				String html = CyActivator.communication.sendForm(URL+"pages/tmp/"+id+"/result.php?tmpdat=result", setPostParameters("pathway///"+pathwayIndex+"<SEPARATOR>tissuevis///0"));

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the graph computation."); return;}

				if (html == null) {
					throw new ExperimentException("EnrichNet pathway result page is empty.");
				}
				logger.info("Form response length: {}", html.length());

				taskMonitor.setStatusMessage("Downloading results");

				// Conversion identifiers
				logger.info("Reading ENSEMBL identifiers.");
				String graphResultHtml = CyActivator.communication.readFile(URL+"pages/tmp/"+id+"/vis"+pathwayIndex+".php");
				HashMap<String, String> conversionEnsemblId = new JEPETTOParser(graphResultHtml).readDataFromVisualGraph();

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the graph computation."); return;}

				// Edges
				logger.info("Reading pathway edges.");
				String edgesResult = CyActivator.communication.readFile(URL+"pages/tmp/"+id+"/network"+pathwayIndex+".lst");
				ArrayList<String[]> edges = new JEPETTOParser(edgesResult).readDataFromLst();

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the graph computation."); return;}

				// Category
				logger.info("Reading pathway node attributes.");
				String categoryResult = CyActivator.communication.readFile(URL+"pages/tmp/"+id+"/network"+pathwayIndex+"_node_attributes.nda");
				HashMap<String, String> category = new JEPETTOParser(categoryResult).readDataFromNda();

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the graph computation."); return;}

				// if pathway expansion exists add extra nodes to the graph
				String pathwayID = enrichnetPathways.get(pathwayName);
				if (pathExpand.pathwaysMap.containsKey(pathwayID)) {
					logger.info("Getting pathway expansion.");
						new JEPETTOPathExpandMethods(database, pathwayID, conversionEnsemblId, edges, category, taskMonitor);
				}

				// generate a network view with input gene set, pathway and extension
				logger.info("Generating network.");
				new JEPETTOGenerateCytoscapeNetwork(pathwayName, edges, category, conversionEnsemblId);
			}
		};

		TaskIterator tIt = new TaskIterator(graphTask);
		CyActivator.taskManager.execute(tIt);
	}

	/**
	 * Retrieve data for regression plot
	 * @param htmlPage
	 * @return a pair of regression values
	 */
	private Double[] get2DPlotValues(String htmlPage) {
		return new JEPETTOParser(htmlPage).readCorrelationValues();
	}

	/**
	 * Get the regression plot
	 * @param id
	 * @return regression plot
	 */
	private JPanel get2DPlot(String id) {
		JPanel panelPlot = new JPanel(new BorderLayout());
		Image image;

		try {
			image = CyActivator.communication.retrieveImage(URL+"pages/tmp/"+id+"/2Dplot.png");
		} catch(MalformedURLException e) {
			logger.error("Regression plot URL malformed.");
			return panelPlot;
		} catch(IOException e) {
			logger.error("Regression plot not read: {}", e.getMessage());
			return panelPlot;
		}

		ImageIcon icon = new ImageIcon(image);
		JLabel labelPlot = new JLabel(icon);
		panelPlot.add(labelPlot, BorderLayout.CENTER);
		return panelPlot;
	}

	/**
	 * Get tissue-specificity values
	 * @param id
	 * @return table of tissue-specificity values
	 */
	private JTable getTissueSpecificities(String id)
	throws ExperimentException {

		String tissuesFile = CyActivator.communication.readFile(URL+"pages/tmp/"+id+"/tissue_specificity_matrix.txt");
		String[][] resultTissues = new JEPETTOParser(tissuesFile).readDataFromTissueSpecificity();
		String[] colNames = {
				"", "atrioventricular node", "trigeminal ganglion", "globus pallidus", "ciliary ganglion", "superior cervical ganglion", "tongue", "testis - interstitial", "testis", "skeletal muscle", "dorsal root ganglion", "lymph node", "bone marrow", "thymus", "trachea", "tonsil", "salivary gland", "appendix", "prefrontal cortex", "amygdala", "subthalamic nucleus", "testis - seminiferous tubule", "pons", "temporal lobe", "skin", "parietal lobe", "caudate nucleus", "liver", "thalamus", "cerebellum", "cingulate cortex", "kidney", "cerebellum peduncles", "occipital lobe", "hypothalamus", "adrenal gland", "medulla oblongata", "heart", "fetal thyroid", "adrenal cortex", "thyroid", "whole blood", "white blood cells", "fetal brain", "fetal liver", "tumor", "placenta", "smooth muscle", "olfactory bulb", "ovary", "lung", "whole brain", "uterus corpus", "uterus", "spinal cord", "prostate", "pancreas", "fetal lung", "adipose tissue", "pituitary", "beta cell islets"
		};
		// Create table of values
		JTable tissueTable = new JEPETTOTable(resultTissues, colNames, colNames, false, null).getCreatedTable();
		tissueTable.setFillsViewportHeight(false);
		tissueTable.setAutoscrolls(true);
		tissueTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		return tissueTable;
	}

	/**
	 * Read similarity ranking data
	 * @param id
	 * @param threshold
	 * @param database
	 * @return index -> pathway name map
	 */
	private void prepareRanking(String id, Double threshold, final String database)
	throws ExperimentException {

		// read and parse ranking data
		String rankingFile = CyActivator.communication.readFile(URL+"pages/tmp/"+id+"/enrichnet_ranking_table.txt");
		String[][] ranking = new JEPETTOParser(rankingFile).readDataFromRanking();

		// filter pathway names
		Pattern p = getPathwayPattern(database);
		for (int counter = 0 ; counter < ranking.length ; ++counter) {
			// FIXME: why pathway list is passed as argument here?
			//        getter function should just return the data,
			//        then in the next line data should be added to the list.
			pathwaysList = getPathwayIdName(pathwaysList, ranking, counter, p, database);
		}

		// create ranking table
		String[] headers = {"Pathway or Process", "XD-score", "q-value", "Overlap/Size"};
		String[] tooltips = {"Pathway/process from reference database",
			"Significance of interaction network distance between input gene set and pathway (XD-score)",
			"Significance of overlap (Fisher-test q-value)",
			"Number of overlapped pathway genes vs pathway size"};
		rankingTable = getTable(ranking, headers, tooltips, false, false, 4, threshold);
		networkComputationIndexesMap = getNetworkComputationsIndexes(rankingTable);

		// listen to mouse click event on pathway list, compute graph on click
		rankingTable.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					final JTable target = (JTable)e.getSource();
					int viewRow = target.getSelectedRow();
					final int modelRow = target.convertRowIndexToModel(viewRow);
					computeGraph(idEnrichNet, networkComputationIndexesMap.get(target.getModel().getValueAt(modelRow, 0)), (String) target.getModel().getValueAt(modelRow, 0), pathwaysList, database);
				}
			}
		});
	}

	/**
	 * Built pathways/processes name display
	 * @param map
	 * @param table
	 * @param index
	 * @param p
	 * @param db
	 * @return map of displayed and corresponding names
	 */
	private HashMap<String, String> getPathwayIdName(HashMap<String, String> map, String[][] table, int index, Pattern p, String db) {
		Matcher m = p.matcher(table[index][0]);
		if (m.find()) {
			if (db.matches("KEGG") | db.matches("BioCarta") | db.matches("InterPro") | db.matches("GO Biological Process") | db.matches("GO Molecular Function") | db.matches("GO Cellular Component") | db.matches("NCI Pathway Interaction")) {
				table[index][0] = m.group(2);
				map.put(m.group(2), "("+m.group(1)+")");
			} else {
				map.put(table[index][0], "("+table[index][0]+")");
			}
		}
		return map;
	}

	/**
	 * Built pattern for pathways/processes name filtering
	 * @param db
	 * @return appropriate database format pattern
	 */
	private Pattern getPathwayPattern(String db) {
		Pattern p = null;
		if (db.matches("KEGG") | db.matches("BioCarta") | db.matches("InterPro") | db.matches("NCI Pathway Interaction")) {
			p = Pattern.compile("^(.+?):(.+)$");
		} else if (db.matches("GO Biological Process") | db.matches("GO Molecular Function") | db.matches("GO Cellular Component")) {
			p = Pattern.compile("^(GO:[0-9]{7})~(.+)$");
		} else { // Wiki-pathway / CORUM / Reactome
			p = Pattern.compile("^(.+?)$");
		}
		return p;
	}
}
