/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.commons.lang.StringUtils;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.model.CyTableUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JEPETTOEnrichNetPanel extends JPanel implements ActionListener {

	/*******************
	 *    ATTRIBUTES   *
	 *******************/

	private final Logger logger = LoggerFactory.getLogger(getClass());

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 2643933736861486942L;

	// UI components
	private static JComboBox molecularNetworkBox = null;
	private static JComboBox idFormatBox = null;
	private static JComboBox annotationDbBox = null;

	private static String chosenNetwork = "Default (STRING)";

	private static JCheckBox isTissueSpecific = null;
	private static JTextArea idArea = null;

	private static JButton inputSetStartButton = new JButton("Use gene list");
	protected static JButton selectedNodesStartButton = new JButton("Use selected nodes");


	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param mainPanel
	 */
	public JEPETTOEnrichNetPanel() {
		super();
		setGeneralLayout();
	}


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Create the general layout of the GUI
	 */
	private void setGeneralLayout() {
		this.setLayout(new BorderLayout(2,6));

		JPanel northPanel = setNorthPanel();
		JPanel centerPanel = setCenterPanel();
		JPanel southPanel = setSouthPanel();

		this.add(northPanel, BorderLayout.NORTH);
		this.add(centerPanel, BorderLayout.CENTER);
		this.add(southPanel, BorderLayout.SOUTH);
	}

	/**
	 * Create the north panel of the GUI
	 * to set up the general parameters
	 * @param panel
	 */
	private JPanel setNorthPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(5,2,2,4));

		// network
		JLabel molecularNetworkLabel = new JLabel("<html><b>Molecular network:</b></html>");
		panel.add(molecularNetworkLabel);	// west
		setupMolecularNetworksBox();
		molecularNetworkBox.addActionListener(this);
		panel.add(molecularNetworkBox);	// east

		// ID format
		JLabel idFormatLabel = new JLabel("<html><b>Identifier format:</b></html>");
		panel.add(idFormatLabel);	// west
		setupIdFormatBox();
		panel.add(idFormatBox);	// east

		// database
		JLabel annotationDbLabel = new JLabel("<html><b>Reference database:</b></html>");
		panel.add(annotationDbLabel);
		setupAnnotationDbBox();
		panel.add(annotationDbBox);

		// tissues
		JLabel isTissueSpecificLabel = new JLabel("<html><b>Tissue-specificity:</b></html>");
		panel.add(isTissueSpecificLabel);	// west

		isTissueSpecific = new JCheckBox();
		String isTissueSpecificText = "<html><it>activate (longer time)</it></html>";
		isTissueSpecific.setText(isTissueSpecificText);
		isTissueSpecific.setToolTipText("Optional: predict values at the tissue level.");
		panel.add(isTissueSpecific);	// east

		// advanced options
		panel.add(new JPanel());
		final JButton optionsButton = new JButton("Advanced settings");
		optionsButton.setToolTipText("Change path expansion criteria.");
		optionsButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				new JEPETTOOptionDialog();
			}
		});
		panel.add(optionsButton);

		return panel;
	}

	/**
	 * Create the geneset panel of the GUI
	 * @param panel
	 */
	private JPanel setCenterPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout(2,4));

		// Gene set area label
		JLabel geneSetLabel = new JLabel("<html><b>Gene/protein set</b> (1 per line):</html>");
		panel.add(geneSetLabel, BorderLayout.NORTH);

		// Gene set area
		idArea = new JTextArea();
		idArea.setToolTipText("Add gene/protein names to the list or select nodes in the current network.");

		idArea.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void removeUpdate(DocumentEvent e) {
				if (idArea.getText().isEmpty()) {
					inputSetStartButton.setEnabled(false);
				}
			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				inputSetStartButton.setEnabled(true);
			}

			@Override
			public void changedUpdate(DocumentEvent arg0) {
				if (idArea.getText().isEmpty()) {
					inputSetStartButton.setEnabled(false);
				} else {
					inputSetStartButton.setEnabled(true);
				}
			}
		});

		JScrollPane geneSetScroll = new JScrollPane(idArea);
		panel.add(geneSetScroll, BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Fill in annotation database box
	 * @param box
	 * @return annotation database box
	 */
	private static void setupAnnotationDbBox() {
		//String[] items = {"KEGG", "BioCarta", "InterPro", "GO Biological Process",
		//	"GO Molecular Function", "GO Cellular Component"};


		String[] items = {"KEGG", "BioCarta", "InterPro", "GO Biological Process",
			"GO Molecular Function", "GO Cellular Component", "Wiki Pathways", "NCI Pathway Interaction", "CORUM complexes", "Reactome"};

		annotationDbBox = new JComboBox(items);
		annotationDbBox.setToolTipText("Choose a reference database of pathways/processes.");
		annotationDbBox.setSelectedIndex(0);
	}

	/**
	 * Create the south panel of the GUI
	 * to start an analysis
	 * @param panel
	 */
	private JPanel setSouthPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(1, 2));

		selectedNodesStartButton = new JButton("Use selected nodes");
		selectedNodesStartButton.setToolTipText("Start the analysis using selected nodes in the current network.");
		selectedNodesStartButton.setEnabled(false);

		selectedNodesStartButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				LinkedList<String> labels = getLabelsOfSelectedNodes();
				if (labels == null) {
					return;
				}

				idArea.setText(StringUtils.join(labels, System.getProperty("line.separator")));
				idArea.repaint();

				if (isInputSizeOK(labels.size())) {
					String inputSet = StringUtils.join(labels, "\r\n");
					runExperiment(inputSet);
				}
			}
		});

		inputSetStartButton = new JButton("Use gene list");
		inputSetStartButton.setToolTipText("Start the analysis using names on the list.");
		inputSetStartButton.setEnabled(false);

		inputSetStartButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				String text = idArea.getText();
				String[] lines = text.split("\\r?\\n");

				if (isInputSizeOK(lines.length)) {
					String inputSet = StringUtils.join(lines, "\r\n");
					runExperiment(inputSet);
				}
			}
		});

		panel.add(inputSetStartButton);
		panel.add(selectedNodesStartButton);

		return panel;
	}


	/**
	 * Check whether the submitted input set size is in range [10,2000].
	 * @param inputSet
	 * @return true if size within limits, otherwise false
	 */
	private boolean isInputSizeOK(int size) {
		if (size > 2000) {
			JOptionPane.showMessageDialog(null, "Please reduce the input set size to 2000 genes (currently "
				+ Integer.toString(size) + ").", "Size limit reached!", JOptionPane.WARNING_MESSAGE);
			return false;
		}
		if (size < 10) {
			JOptionPane.showMessageDialog(null, "Please input at least 10 genes.", "Input set too small!", JOptionPane.WARNING_MESSAGE);
			return false;
		}
		return true;
	}


	private void runExperiment(String inputSet) {
		new JEPETTOEnrichNetMethods(chosenNetwork, (String) idFormatBox.getSelectedItem(),
			isTissueSpecific.isSelected(), inputSet, (String) annotationDbBox.getSelectedItem());
	}

	/**
	 * Set up the molecular network combobox
	 */
	private void setupMolecularNetworksBox() {
		String[] items = {"Default (STRING)", "Sparse (STRING 900)",
			"Sparse (Bossi & Lehner)", "Integrative physical network (PSICQUIC)",
			"Upload your own network"};

		molecularNetworkBox = new JComboBox(items);
		molecularNetworkBox.setToolTipText("Choose an interaction network from the list or upload your own.");
		molecularNetworkBox.setSelectedIndex(0);
	}

	/**
	 * Set up the identifiers format combobox
	 */
	private static void setupIdFormatBox() {
		String[] items = {"HGNC SYMBOL", "ENSEMBL ID", "ENTREZ GENE",
			"UNIPROT/SWISSPROT ID", "UNIPROT/SWISSPROT ACC", "REFSEQ DNA",
			"PROTEIN ID", "Affymetrix HC G110", "Affymetrix HG Focus",
			"Affymetrix HG U133 plus 2", "Affymetrix HG U133a",
			"Affymetrix HG U133a 2", "Affymetrix HG U133b",
			"Affymetrix HG U95a", "Affymetrix HG U95av2", "Affymetrix HG U95b",
			"Affymetrix HG U95c", "Affymetrix HG U95d", "Affymetrix HG U95e",
			"Affymetrix HuEx-1_0-st-v2", "Affymetrix HuGene-1_0-st-v1",
			"Affymetrix HuGene FL", "Affymetrix U133 X3P", "Agilent CGH 44b",
			"Agilent Wholegenome", "Illumina HumanWG-6 v1",
			"Illumina HumanWG-6 v2", "Illumina HumanWG-6 v3"};

		idFormatBox = new JComboBox(items);
		idFormatBox.setToolTipText("Choose a naming convention used for the input set.");
		idFormatBox.setSelectedIndex(0);
	}

	/**
	 * Reads labels of the currently selected network nodes.
	 */
	protected LinkedList<String> getLabelsOfSelectedNodes() {
		CyNetwork currentNetwork = CyActivator.cyApplicationManagerService.getCurrentNetwork();
		if (currentNetwork == null) {
			JOptionPane.showMessageDialog(null, "Network not found. Please load a network and select nodes first.");
			return null;
		}

		List<CyNode> nodes = CyTableUtil.getNodesInState(currentNetwork, "selected", true);
		if (nodes.size() == 0) {
			JOptionPane.showMessageDialog(null, "There are no nodes selected nodes in the current network view.");
			return null;
		}

		LinkedList<String> labels = new LinkedList<String>();
		for (CyNode node : nodes) {
			String label = currentNetwork.getRow(node).get(CyNetwork.NAME, String.class);
			labels.add(label);
		}

		return labels;
	}

	/**
	 * Listener for user actions
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		// Network import
		if (e.getSource() == molecularNetworkBox) {
			if (molecularNetworkBox.getSelectedItem() == "Upload your own network") {
				Object[] options = {"OK"};

				String message = "This option is time-consuming,"
					+ "please limit your network size to 10,000 interactions.";

				int n = JOptionPane.showOptionDialog(null, message, "Warning: upload network",
					JOptionPane.PLAIN_MESSAGE, JOptionPane.QUESTION_MESSAGE,
					null, options, options[0]);

				if (n == JOptionPane.OK_OPTION) {
					JFileChooser chooseFile = new JFileChooser();
					int fchoose = chooseFile.showOpenDialog(null);

					if (fchoose == JFileChooser.APPROVE_OPTION) {
						String network = chooseFile.getSelectedFile().getAbsolutePath();

						if (checkNetworkFormat(network)) {
							chosenNetwork = network;
						} else {
							molecularNetworkBox.setSelectedIndex(0);

							message = "The format of the uploaded file\n"
								+ "is not supported. Each line should describe\n"
								+ "a single edge as follows:\n"
								+ "<source> <target> (space separated),\n"
								+ "using Ensembl gene identifier format.";

							JOptionPane.showMessageDialog(this,	message,
								"File format not supported", JOptionPane.WARNING_MESSAGE);
						}
					} else {
						molecularNetworkBox.setSelectedIndex(0);
						chosenNetwork = (String) molecularNetworkBox.getSelectedItem();
					}
				}
			} else {
				chosenNetwork = (String) molecularNetworkBox.getSelectedItem();
			}
		}
	}

	/**
	 * Check the user uploaded file format
	 * @param fileName
	 * @return format is appropriate or not
	 */
	private boolean checkNetworkFormat(String fileName) {
		Pattern ensemblPattern = Pattern.compile("ENSG[0-9]{11}");
		BufferedReader br = null;
		boolean isGoodFormat = true;
		try {
			String sCurrentLine;
			br = new BufferedReader(new FileReader(fileName));
			file : while ((sCurrentLine = br.readLine()) != null) {
				String[] interaction = sCurrentLine.split("\t");
				if (interaction.length > 2) {
					isGoodFormat = false;
					break file;
				}
				format : for (String geneId : interaction) {
					Matcher m = ensemblPattern.matcher(geneId);
					if (m.find()) {
						continue format;
					}
					isGoodFormat = false;
					break file;
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				br.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return isGoodFormat;
	}
}
