/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.cytoscape.model.CyEdge;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.model.CyRow;
import org.cytoscape.model.CyTable;
import org.cytoscape.view.layout.CyLayoutAlgorithm;
import org.cytoscape.view.model.CyNetworkView;
import org.cytoscape.view.model.View;
import org.cytoscape.view.presentation.property.BasicVisualLexicon;
import org.cytoscape.work.TaskIterator;

public class JEPETTOGenerateCytoscapeNetwork extends CyActivator {

	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param name
	 * @param edges
	 * @param category
	 * @param attributes
	 */
	public JEPETTOGenerateCytoscapeNetwork(String name,
			ArrayList<String[]> edges,
			HashMap<String, String> category,
			HashMap<String, String> attributes) {
		super();
		CyNetwork network = generateNewNetwork(name);
		implementNetwork(network, edges, category, attributes);
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Add nodes and edges to a network
	 * @param network
	 * @param edges
	 */
	private void addNodesEdges(CyNetwork network, ArrayList<String[]> edges) {
		ArrayList<String> nodes  = new ArrayList<String>();
		HashMap<String, Long> nodeIdList = new HashMap<String, Long>();
		for (String[] s : edges) {
			CyNode n1 = null;
			CyNode n2 = null;
			if (!nodes.contains(s[0])) {
				nodes.add(s[0]);
				n1 = network.addNode();
				nodeIdList.put(s[0], n1.getSUID());
				network.getRow(n1).set(CyNetwork.NAME, s[0]);
			} else {
				n1 = network.getNode(nodeIdList.get(s[0]));
			}
			if (s.length > 1) {
				if (!nodes.contains(s[1])) {
					nodes.add(s[1]);
					n2 = network.addNode();
					nodeIdList.put(s[1], n2.getSUID());
					network.getRow(n2).set(CyNetwork.NAME, s[1]);
				} else {
					n2 = network.getNode(nodeIdList.get(s[1]));
				}
				if (!network.containsEdge(n1, n2) & !network.containsEdge(n2, n1)) {
					network.addEdge(n1, n2, false);
				}
			}
		}
	}

	/**
	 * Custom nodes attributes
	 * @param network
	 * @param category
	 * @param attributes
	 * @return cytoscape table panel table
	 */
	private CyTable customNodeAttributes(CyNetwork network, HashMap<String, String> category, HashMap<String, String> attributes) {
		// create table
		CyTable myTable = network.getDefaultNodeTable();
		// Create new columns
		String labelAttributeColumn = "Label";
		String ensemblLabelColumn = "Ensembl";
		myTable.createColumn(labelAttributeColumn, String.class, false);
		myTable.createColumn(ensemblLabelColumn, String.class, false);
		for (CyRow row : myTable.getAllRows()) {
			Map<String, Object> rowMap = row.getAllValues();
			row.set(labelAttributeColumn, category.get(rowMap.get("name")));
			row.set(ensemblLabelColumn, attributes.get(rowMap.get("name")));
		}
		return myTable;
	}

	/**
	 * Apply custom style to the network view
	 * @param network
	 * @param myTable
	 * @return customized network view
	 */
	private CyNetworkView customNetworkView(CyNetwork network, CyTable myTable) {
		CyNetworkView netView = CyActivator.networkViewFactory.createNetworkView(network);
		// Custom nodes
		for (View<CyNode> node : netView.getNodeViews()) {
			String label = (String) myTable.getRow(node.getModel().getSUID()).getRaw("Label");
			node.setVisualProperty(BasicVisualLexicon.NODE_TOOLTIP, (String) myTable.getRow(node.getModel().getSUID()).getRaw("name"));
			if (label.matches("geneset/added")) {
				node.setVisualProperty(BasicVisualLexicon.NODE_BORDER_WIDTH, node.getVisualProperty(BasicVisualLexicon.NODE_HEIGHT)/2d);
				node.setVisualProperty(BasicVisualLexicon.NODE_BORDER_PAINT, new Color(255, 83, 23));
				node.setVisualProperty(BasicVisualLexicon.NODE_FILL_COLOR, Color.lightGray);
			} else if (label.matches("geneset")) {
				node.setVisualProperty(BasicVisualLexicon.NODE_FILL_COLOR, Color.lightGray);
			} else if (label.matches("overlap")) {
				node.setVisualProperty(BasicVisualLexicon.NODE_FILL_COLOR, new Color(0, 87, 150));
			} else if (label.matches("pathway/process")) {
				node.setVisualProperty(BasicVisualLexicon.NODE_FILL_COLOR, new Color(105, 171, 41));
			} else {
				node.setVisualProperty(BasicVisualLexicon.NODE_FILL_COLOR, new Color(255, 83, 23));
			}
		}
		// Custom edges
		for (View<CyEdge> edge : netView.getEdgeViews()) {
			String sourceLabel = (String) myTable.getRow(edge.getModel().getSource().getSUID()).getRaw("Label");
			String targetLabel = (String) myTable.getRow(edge.getModel().getTarget().getSUID()).getRaw("Label");
			edge.setVisualProperty(BasicVisualLexicon.EDGE_TOOLTIP, (String) myTable.getRow(edge.getModel().getSource().getSUID()).getRaw("name")+" -> "+(String) myTable.getRow(edge.getModel().getTarget().getSUID()).getRaw("name"));
			if ((sourceLabel.matches("added") & !targetLabel.matches("added")) | (!sourceLabel.matches("added") & targetLabel.matches("added"))) {
				edge.setVisualProperty(BasicVisualLexicon.EDGE_UNSELECTED_PAINT, new Color(255, 83, 23));
			} else if ((sourceLabel.matches("pathway/process") & targetLabel.matches("overlap")) | (sourceLabel.matches("overlap") & targetLabel.matches("pathway/process")) | (sourceLabel.matches("pathway/process") & targetLabel.matches("geneset")) | (sourceLabel.matches("geneset") & targetLabel.matches("pathway/process"))) {
				edge.setVisualProperty(BasicVisualLexicon.EDGE_UNSELECTED_PAINT, new Color(105, 171, 41));
			} else {
				edge.setVisualProperty(BasicVisualLexicon.EDGE_UNSELECTED_PAINT, Color.darkGray);
			}
		}
		return netView;
	}

	/**
	 * Implement Cytoscape network
	 * @param network
	 * @param edges
	 * @param category
	 * @param attributes
	 */
	private void implementNetwork(CyNetwork network,
			ArrayList<String[]> edges,
			HashMap<String, String> category,
			HashMap<String, String> attributes) {
		addNodesEdges(network, edges);
		CyTable nodeAttributeTable = customNodeAttributes(network, category, attributes);

		// Add the network
		CyActivator.cyNetworkManagerServiceRef.addNetwork(network);
		CyNetworkView netView = customNetworkView(network, nodeAttributeTable);

		//FIXME: what if netView is null? causes NullPointerException later

		// Apply force-directed layout
		CyLayoutAlgorithm layoutAlgorithm = CyActivator.layoutAlgoManager.getLayout("force-directed");
		TaskIterator tasks = layoutAlgorithm.createTaskIterator(netView, layoutAlgorithm.createLayoutContext(), CyLayoutAlgorithm.ALL_NODE_VIEWS, null);
		CyActivator.taskManager.execute(tasks);
		CyActivator.networkViewManager.addNetworkView(netView);
	}

	/**
	 * Create Cytoscape network
	 * @param name
	 * @return
	 */
	private static CyNetwork generateNewNetwork(String name) {
		CyNetwork myNet = networkFactory.createNetwork();
		myNet.getRow(myNet).set(CyNetwork.NAME, name);
		return myNet;
	}
}
