/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ToolTipManager;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.CustomXYToolTipGenerator;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

public class JEPETTOGenerateScatterPlot {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	// Constructor attributes: axis
	private final HashMap<String, ArrayList<ArrayList<String>>> values;	// list of values of the comparative analysis
	private final String x;	// x-axis to plot
	private final String y;	// y-axis to plot

	// Panel containing the plot
	private static ChartPanel chartPanel;

	// Correspondance map
	private static final HashMap<String, Integer> valId = new HashMap<String, Integer>();
	static {
		valId.put("Degree", 1);
		valId.put("Clustering Coefficient", 2);
		valId.put("Shortest Path Length", 3);
		valId.put("Node Betweenness", 4);
		valId.put("Eigenvector Centrality", 5);
		valId.put("Number of Genes", 6);
		valId.put("Score", 7);
	}



	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param values
	 * @param x
	 * @param y
	 */
	public JEPETTOGenerateScatterPlot(HashMap<String, ArrayList<ArrayList<String>>> values, String x, String y){
		this.values = values;
		this.x = x;
		this.y = y;

		JFreeChart chart = ChartFactory.createScatterPlot(
				setTitle(), // chart title
				this.x, // x-axis label
				this.y, // y-axis label
				getDataset(), // dataset to plot
				PlotOrientation.VERTICAL, // orientation
				true, // show legends
				true, // show tooltips
				false // configure chart to generate urls
				);

		chart.setTitle(new TextTitle(setTitle(), new Font("SansSerif", Font.BOLD, 13)));
		createCustomTooltip(chart);

		JEPETTOGenerateScatterPlot.setChartPan(new ChartPanel(chart));
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Generate a specific tooltip for the plot components
	 * @param chart
	 */
	private void createCustomTooltip(JFreeChart chart) {
		XYPlot plot = (XYPlot) chart.getPlot();
		XYItemRenderer renderer = plot.getRenderer();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.darkGray);
		CustomXYToolTipGenerator tooltip = new CustomXYToolTipGenerator();
		int i = 0;
		ToolTipManager.sharedInstance().setDismissDelay(30000);
		ToolTipManager.sharedInstance().setInitialDelay(0);
		for (Map.Entry<String, ArrayList<ArrayList<String>>> entry : this.values.entrySet()) {
			ArrayList<String> tips = new ArrayList<String>();
			String category = entry.getKey();
			for (ArrayList<String> l : entry.getValue()) {
				tips.add("<html><b>Class:</b> " + category + "<br>"
						+ "<b>Entity:</b> " + l.get(0) + "<br>"
						+ "<b>Coordinates:</b> (" + l.get(valId.get(this.x)) + ", " + l.get(valId.get(this.y)) + ")</html>");
			}
			tooltip.addToolTipSeries(tips);
			renderer.setSeriesToolTipGenerator(i, tooltip);
			i++;
		}
	}

	/**
	 * Create the dataset for the plot
	 * @return data collection
	 */
	private XYSeriesCollection getDataset(){
		XYSeriesCollection collection = new XYSeriesCollection();
		for (Map.Entry<String, ArrayList<ArrayList<String>>> entry : this.values.entrySet()) {
			XYSeries s = new XYSeries(entry.getKey(), false);
			for (ArrayList<String> l : entry.getValue()) {
				s.add(new Double(l.get(valId.get(this.x)).replace(",", ".")), new Double(l.get(valId.get(this.y)).replace(",", ".")));
			}
			collection.addSeries(s);
		}
		return collection;
	}

	/**
	 * Set the plot title
	 * @return the title of the plot
	 */
	private String setTitle(){
		return "Comparative analysis: "+this.y+" against "+this.x;
	}

	/**
	 * Get the panel containing the plot
	 * with specific dimensions
	 * @return the plot panel
	 */
	protected ChartPanel getChartPanel() {
		return chartPanel;
	}

	/**
	 * Set the plot panel to the generated chart
	 * @param chartPanel
	 */
	private static void setChartPan(ChartPanel chartPanel) {
		JEPETTOGenerateScatterPlot.chartPanel = chartPanel;
	}
}
