/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JTable;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public abstract class JEPETTOGenericServerMethods {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	@SuppressWarnings("unused")
	private final Logger logger = LoggerFactory.getLogger(getClass());

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 6072221589011896171L;


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Get the ID of the session
	 * @param htmlPage
	 * @return the session id
	 */
	protected static String getID(String html, String regexp) {
		Pattern pattern = Pattern.compile(regexp);
		Matcher matcher = pattern.matcher(html);
		String id = null;
		if (matcher.find()) {
			id = matcher.group(1);
		}
		return id;
	}

	/**
	 * Set the post parameter request
	 * @return post parameters
	 */
	protected static List<NameValuePair> setPostParameters(String params) {
		List<NameValuePair> paramList = new ArrayList<NameValuePair>();
		for (String s : params.split("<SEPARATOR>")) {
			if (s.split("///").length > 1) {
				paramList.add(new BasicNameValuePair(s.split("///")[0], s.split("///")[1]));
			} else {
				paramList.add(new BasicNameValuePair(s.split("///")[0], ""));
			}
		}
		return paramList;
	}

	/**
	 * Compute the output table from result files
	 * @param url
	 * @param file
	 * @param parsingMethod
	 * @param colHeaders
	 * @param colTooltips
	 * @param isStat
	 * @param resizeModeValue
	 * @return data table
	 */
	protected static JTable getTableFromFile(String url, String parsingMethod,
		String[] colHeaders, String[] colTooltips, boolean isStat, int resizeModeValue)
		throws ExperimentException {

		String fileString = CyActivator.communication.readFile(url);
		JEPETTOParser parser = new JEPETTOParser(fileString);

		Method method = null;
		try {
			method = JEPETTOParser.class.getMethod(parsingMethod, (Class<?>[]) null);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		}

		String[][] fileData = null;
		try {
			fileData = (String[][]) method.invoke(parser);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		JTable table = new JEPETTOTable(fileData, colHeaders, colTooltips, isStat, null).getCreatedTable();
		table.setFillsViewportHeight(false);
		table.setAutoscrolls(true);
		table.setAutoResizeMode(resizeModeValue);
		return table;
	}

	/**
	 * Compute the output table from some data
	 * @param data
	 * @param colHeaders
	 * @param colTooltips
	 * @param isStat
	 * @param resizeModeValue
	 * @return data table
	 */
	protected static JTable getTable(String[][] data,
			String[] colHeaders,
			String[] colTooltips,
			boolean isStat,
			boolean isTopology,
			int resizeModeValue,
			double significanceThreshold) {
		JTable table = new JEPETTOTable(data, colHeaders, colTooltips, isStat, significanceThreshold).getCreatedTable();
		table.setFillsViewportHeight(false);
		table.setAutoscrolls(true);
		table.setAutoResizeMode(resizeModeValue);
		if (isTopology) {
			table.getRowSorter().toggleSortOrder(table.getColumnCount()-1);
		}
		return table;
	}

	/**
	 * Provides a generic format for parameters to send the webservers
	 * @param paramNames
	 * @param paramValues
	 * @param paramDefaultLists
	 * @return a string of parameters
	 */
	protected static String getPostParameters(String[] paramNames, String[] paramValues, String[] paramDefaultLists) {
		String parameterStringRequest = "";
		int index = 0;
		while (index < paramNames.length) {
			boolean lastParameter = (index+1 == paramNames.length) ? true : false;
			if (paramDefaultLists[index] == null) {
				parameterStringRequest += (lastParameter) ? paramNames[index]+"///"+paramValues[index] : paramNames[index]+"///"+paramValues[index]+"<SEPARATOR>";
			} else if (paramNames[index] == null) {
				HashMap<String, String[]> parameterMap = new HashMap<String, String[]>();
				for (String elt : paramDefaultLists[index].split("\\+")) {
					parameterMap.put(elt.split("///")[0], elt.split("///")[1].split("_"));
				}
				for (String elt : paramValues[index].split("///")) {
					parameterStringRequest += (lastParameter) ? parameterMap.get(elt)[0]+"///"+parameterMap.get(elt)[1] : parameterMap.get(elt)[0]+"///"+parameterMap.get(elt)[1]+"<SEPARATOR>";
				}
			} else {
				HashMap<String, String> parameterMap = new HashMap<String, String>();
				for (String elt : paramDefaultLists[index].split("\\+")) {
					parameterMap.put(elt.split("///")[0], elt.split("///")[1]);
				}
				parameterStringRequest += (lastParameter) ? paramNames[index]+"///"+parameterMap.get(paramValues[index]) : paramNames[index]+"///"+parameterMap.get(paramValues[index])+"<SEPARATOR>";
			}
			index++;
		}
		return parameterStringRequest;
	}
}
