/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JEPETTOParser {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	@SuppressWarnings("unused")
	private final Logger logger = LoggerFactory.getLogger(getClass());

	// Html page to parse
	private static String htmlcode;


	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param htmlcode
	 */
	public JEPETTOParser(String htmlcode){
		JEPETTOParser.setHtmlcode(htmlcode);
	}


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Enrichnet uploaded set medians
	 * @return uploaded set values
	 */
	public Double[] readDataFromRankingtop() {
		Double[] values = new Double[7];
		Pattern p = Pattern.compile("<table.+?</tr>.+?</td><td.+?>(.+?)</td><td.+?>(.+?)</td><td.+?>(.+?)</td><td.+?>(.+?)</td><td.+?>(.+?)</td><td.+?>(.+?)</td></tr></table>");
		Matcher m = p.matcher(getHtmlcode());
		if (m.find()){
			values[0] = Double.parseDouble(m.group(1));
			values[1] = Double.parseDouble(m.group(2));
			values[2] = Double.parseDouble(m.group(3));
			values[3] = Double.parseDouble(m.group(4));
			values[4] = Double.parseDouble(m.group(5));
			values[5] = Double.parseDouble(m.group(6));
			values[6] = 0.0;
		}
		return values;
	}

	/**
	 * Pathway/process plot values
	 * @return plot values
	 */
	public Map<Integer, String> readLabelFromSvgplot(){
		Map<Integer, String> mapLabel = new HashMap<Integer, String>();
		Pattern p = Pattern.compile("dlabs\\.push\\(\"(.+?)\"\\)");
		Matcher m = p.matcher(getHtmlcode());
		int i = 1;
		while (m.find()) {
			mapLabel.put(i, m.group(1));
			i++;
		}
		return mapLabel;
	}

	/**
	 * Merge EnrichNet result values
	 * @param v1
	 * @param v2
	 * @return merges values
	 */
	public static Map<String[], Double[]> mergeSvgplotTop5(Map<String[],Double[]> v1, Map<Integer, String> v2) {
		Map<String[], Double[]> values = new HashMap<String[], Double[]>();
		for(String[] s : v1.keySet()) {
			String[] smap = {s[1], v2.get(Integer.parseInt(s[0]))};
			values.put(smap, v1.get(s));
		}
		return values;
	}

	/**
	 * Top similarity ranking
	 * @return comparative similarity values
	 */
	public Map<String[],Double[]> readDataFromTop5(){
		Map<String[],Double[]> values = new HashMap<String[],Double[]>();
		Pattern p = Pattern.compile("<table.+?</tr></thead>(<tr>.+?</tr>)</table>");
		Pattern pr = Pattern.compile("<tr>(.+?)</tr>");
		Pattern pi = Pattern.compile("id=\'clickdata(\\d+?)\'");
		Pattern pc = Pattern.compile("\'>((.+?)(\\s\\(.+?\\)?)?)<");
		Matcher m = p.matcher(getHtmlcode());
		if (m.find()){
			String table = m.group(1);
			Matcher mr = pr.matcher(table);
			while(mr.find()){
				String row = mr.group(1);
				String[] id = new String[2];
				// match id ref
				Matcher mi = pi.matcher(row);
				if (mi.find()){
					id[0] = mi.group(1);
				} else {
					id[0] = null;
				}
				// match values
				Matcher mc = pc.matcher(row);
				Double[] rowval = new Double[7];
				int i = 0;
				while(mc.find()){
					if (i == 0){
						id[1] = mc.group(2);
					} else {
						rowval[i-1] = Double.parseDouble(mc.group(2));
					}
					i++;
				}
				values.put(id, rowval);
			}
		}
		return values;
	}

	/**
	 * Data extracted from path/process visualization
	 * @return network data
	 */
	public HashMap<String, String> readDataFromVisualGraph() {
		HashMap<String, String> values = new HashMap<String, String>();
		Pattern p = Pattern.compile("Identifier\\s/\\sgene\\snames.+?<table.+?>(.+?)</table>");
		Pattern p1 = Pattern.compile("<tr><td><span.+?>(.+?)</span>\\s?</td><td.+?>(.+?)<.+?</tr>");
		Matcher m = p.matcher(getHtmlcode());
		if (m.find()) {
			String conversionTable = m.group(1);
			Matcher m1 = p1.matcher(conversionTable);
			while (m1.find()) {
				String ensembl = m1.group(1);
				String current = m1.group(2);
				values.put(current, ensembl);
			}
		}
		return values;
	}

	/**
	 * Nodes in network
	 * @return nodes
	 */
	public ArrayList<String[]> readDataFromLst() {
		String[] rowValues = getHtmlcode().split("\\n");
		ArrayList<String[]> values = new ArrayList<String[]>();
		for (String s : rowValues) {
			values.add(s.split("\\s"));
		}
		return values;
	}

	/**
	 * Node attributes
	 * @return network node attributes
	 */
	public HashMap<String, String> readDataFromNda() {
		String[] rowValues = getHtmlcode().split("\\n");
		HashMap<String, String> values = new HashMap<String, String>();
		int ix = 0;
		for (String s : rowValues) {
			if (ix == 0) {
				ix++;
				continue;
			}
			String[] line = s.split("\\s");
			if (line.length == 3) values.put(line[0], line[2]);
		}
		return values;
	}

	/**
	 * Tissue-specificity analysis
	 * @return tissue values
	 */
	public String[][] readDataFromTissueSpecificity() {
		String[] rowValues = getHtmlcode().split("\\n");
		ArrayList<String[]> values = new ArrayList<String[]>();
		int index = 0;
		for (String s : rowValues) {
			String[] vals = s.split("\\t");
			if (index == 0) {
				String[] firstLine = new String[vals.length+1];
				firstLine[0] = "";
				int i = 0;
				while (i < vals.length) {
					firstLine[i+1] = vals[i];
					i++;
				}
				values.add(firstLine);
				index++;
			} else {
				values.add(vals);
			}
		}
		String[][] valuesTable = new String[values.size()-1][61];
		int row = 0;
		for (String[] s1 : values) {
			if (row == 0) {
				row++;
				continue;
			}

			int col = 0;
			for (String s2 : s1) {
				if (col == 0) {
					// get tissue name without enclosing quotation marks
					valuesTable[row-1][col] = s2.substring(1, s2.length()-1);
				} else {
					float value = Float.valueOf(s2.replace(",", "."));
					valuesTable[row-1][col] = String.format("% .2f", Float.valueOf(value));
				}
				col++;
			}
			row++;
		}
		return valuesTable;
	}

	/**
	 * Similarity ranking data
	 * @return similarity values
	 */
	public String[][] readDataFromRanking(){
		ArrayList<String[]> values = new ArrayList<String[]>();
		String cleanFile = getHtmlcode().replaceAll("\"", "");

		String[] lines = cleanFile.split("\\n");
		for (String line : lines) {
			values.add(line.split("\\t"));
		}

		String[][] data = new String[values.size()-1][4];
		for (int i = 1; i < values.size(); ++i) {
			data[i-1][0] = values.get(i)[0];

			String text = values.get(i)[5].trim() + "/" + values.get(i)[4];
			data[i-1][3] = String.format("%7s", text);

			for (int j = 1; j < 3; ++j) {
				float value = Float.parseFloat(values.get(i)[j]);
				data[i-1][j] = String.format("% .5f", value);
			}
		}
		return data;
	}

	/**
	 * Correlation values of the regression plot
	 * @return correlation values
	 */
	public Double[] readCorrelationValues() {
		Pattern p = Pattern.compile("<b>Absolute\\sPearson\\scorrelation.+?</b>.+?<b>(.+?)</b>.+?<b>XD-score\\ssignificance\\sthreshold:.+?</b>.+?<b>(.+?)</b>", Pattern.DOTALL);
		Matcher m = p.matcher(getHtmlcode());
		Double[] values = {0.0, 0.0};
		if (m.find()) {
			String pearsonCorrelationValue = m.group(1);
			String xdScoreCorrelationValue = m.group(2);
			if (!pearsonCorrelationValue.matches("NA")) {
				values[0] = Double.parseDouble(pearsonCorrelationValue);
				values[1] = Double.parseDouble(xdScoreCorrelationValue);
			}
		}
		return values;
	}

	/**
	 * Mapping information of the target set on the background model
	 * @return mapping
	 */
	public String[][] readDataFromMaptable(){
		Pattern p = Pattern.compile("<table.+?</tr>(.+?)</table>",Pattern.DOTALL);
		Matcher m = p.matcher(getHtmlcode());
		ArrayList<String[]> values = new ArrayList<String[]>();
		Pattern pg = Pattern.compile("<td>(.+?)</td>");
		Pattern pc = Pattern.compile("<tr>(.+?)</tr>", Pattern.DOTALL);
		if (m.find()){
			String table = m.group(1);
			Matcher mc = pc.matcher(table);
			while (mc.find()){
				Matcher mg = pg.matcher(mc.group(1));
				ArrayList<String> rowList = new ArrayList<String>();
				while (mg.find()) rowList.add(mg.group(1));
				if (rowList.size() == 3) values.add(new String[]{rowList.get(0), rowList.get(1), rowList.get(2)});
			}
		}
		String[][] data = new String[values.size()][3];
		int row = 0;
		for (String[] s : values) {
			data[row][0] = s[0];
			data[row][1] = s[1];
			data[row][2] = s[2];
			row++;
		}
		return data;
	}

	/**
	 * TopoGSA statistics
	 * @return topological statistics
	 */
	public String[][] readDataFromStat(){
		Pattern p = Pattern.compile("<table.+?><tr>.+?</tr><tr><th>Uploaded\\sgene/protein\\sset</th>(.+?)</tr><tr><th>100\\srandom\\ssimulations\\s\\(mean\\)</th>(.+?)</tr><tr(\\sstyle=\"margin-top:5px;\")?><th(\\sstyle=\"color:\\sdarkgray;\")?>Static.+?</th>(.+?)</tr></table>");
		Matcher m = p.matcher(getHtmlcode());
		String[][] data = new String[3][6];
		data[0][0] = "Data set";
		data[1][0] = "Rand sim";
		data[2][0] = "Entire network";
		if (m.find()) {
			String rUp = m.group(1);
			String rSim = m.group(2);
			String rStat = m.group(5);
			String[] statRows = {rUp, rSim, rStat};
			Pattern pc = Pattern.compile("<th.*?>(.+?)(\\s\\(\\&plusmn;(.+?)\\))?</th>",Pattern.DOTALL);
			int row = 0;
			for (String r : statRows) {
				int col = 1;
				Matcher mc = pc.matcher(r);
				while (mc.find()){
					String val = "";
					val += mc.group(1);
					if (mc.group(3) != null) {
						val += " \u00B1 "+mc.group(3);
					}
					data[row][col] = val;
					col++;
				}
				row++;
			}
		}
		return data;
	}

	/**
	 * Get code to parse
	 * @return html file content
	 */
	private static String getHtmlcode() {
		return htmlcode;
	}

	/**
	 * Set code to parse
	 * @param htmlcode to parse
	 */
	private static void setHtmlcode(String htmlcode) {
		JEPETTOParser.htmlcode = htmlcode;
	}
}
