/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import org.cytoscape.work.TaskMonitor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JEPETTOPathExpandMethods extends JEPETTOGenericServerMethods {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	private static final Logger logger = LoggerFactory.getLogger(JEPETTOPathExpandMethods.class);

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 6470327848588264860L;

	// Server URL
	private static final String URL = "http://www.pathexpand.org/";

	private final HashMap<String, String> conversionEnsemblId;
	private final ArrayList<String[]> edges;
	private final HashMap<String, String> category;

	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param component
	 * @param database
	 * @param pathway
	 * @param isEnrichnetExpansion
	 * @param pathwayName
	 */
	public JEPETTOPathExpandMethods(String database, String pathway,
		HashMap<String, String> conversionEnsemblId, ArrayList<String[]> edges,
		HashMap<String, String> category, TaskMonitor tm)
		throws ExperimentException {

		this.conversionEnsemblId = conversionEnsemblId;
		this.edges = edges;
		this.category = category;

		analysis(database, pathway, tm);
	}


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Start an analysis with PathExpand
	 * @param database
	 * @param pathway
	 * @param isEnrichnetExpansion
	 * @param pathwayName
	 */
	private void analysis(String database, String pathway, TaskMonitor tm)
		throws ExperimentException {

		String[] names = {"database", "genesets", "assoc_thresh", "coverage_thresh", "triangle_thresh"};
		String[] values = {database, pathway, Double.toString(CyActivator.associationValue), Double.toString(CyActivator.coverageValue), Double.toString(CyActivator.triangleValue)};
		String[] defaults = {"BioCarta///biocarta+KEGG///kegg+GO Biological Process///gobp+Reactome///reactome+InterPro///interpro+GO Molecular Function///gomf+GO Cellular Component///gocc", null, null, null, null};
		String parameters = getPostParameters(names, values, defaults);

		tm.setStatusMessage("Waiting for results");

		String html = null;
		try {
			html = CyActivator.communication.sendForm(URL+"index.php?tmpdat=result", setPostParameters(parameters));
		} catch (ExperimentException e) {
			// this is not a critical task
			// is is better to show network without expansion, than to fail
			logger.error("Path expansion not read: {}", e.getMessage());
			return;
		}

		if (html == null) {
			throw new ExperimentException("PathExpand result page is empty.");
		}
		logger.info("Form response length: {}", html.length());

		tm.setStatusMessage("Downloading results");

		String sessionId = getID(html, "([a-z0-9]{5}_[0-9]{10})");
		if (sessionId == null) {
			throw new ExperimentException("PathExpand experiment ID not found.");
		} else {
			logger.info("Experiment ID: {}", sessionId);
		}

		logger.info("Reading ENSEMBL identifiers.");
		String graphResultHtml = CyActivator.communication.readFile(URL+"pages/tmp/"+sessionId+"/result.php");
		HashMap<String, String> conversionEnsemblId = new JEPETTOParser(graphResultHtml).readDataFromVisualGraph();

		logger.info("Reading expansion edges.");
		String edgesResult = CyActivator.communication.readFile(URL+"pages/tmp/"+sessionId+"/network.lst");
		ArrayList<String[]> edges = new JEPETTOParser(edgesResult).readDataFromLst();

		logger.info("Reading expansion node attributes.");
		String categoryResult = CyActivator.communication.readFile(URL+"pages/tmp/"+sessionId+"/network_node_attributes.nda");
		HashMap<String, String> category = new JEPETTOParser(categoryResult).readDataFromNda();

		logger.info("Adding expansion to the network.");
		mergeEnrichnetPathexpandPathway(edges, category, conversionEnsemblId);
	}

	/**
	 * Merge the expansion nodes and edges
	 * @param edges
	 * @param category
	 * @param attributes
	 */
	private void mergeEnrichnetPathexpandPathway(ArrayList<String[]> edges,
		HashMap<String, String> category, HashMap<String, String> attributes) {

		// add new edges
		for (String[] s : edges) {
			if (!this.edges.contains(s)) this.edges.add(s);
		}

		// add new nodes
		for (Entry<String, String> entry : category.entrySet()) {
			if (entry.getValue().equals("added")) {
				if (this.category.containsKey(entry.getKey())) {
					this.category.remove(entry.getKey());
					this.category.put(entry.getKey(), "geneset/added");
				} else {
					this.category.put(entry.getKey(), "added");
				}
			}
			else {
				if (!this.category.containsKey(entry.getKey())) {
					this.category.put(entry.getKey(), "pathway/process");
				}
			}
			String node = entry.getKey();
			this.conversionEnsemblId.put(node, attributes.get(node));

		}
	}
}
