/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;

public class JEPETTOTable extends JTable {

	/*******************
	 *   ATTRIBUTES	*
	 *******************/

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = -7488851086383913825L;

	// Constructor attributes
	private final String[] tooltips;
	private final JTable resultTable;
	private final boolean isStatistics;
	private final Double significanceThreshold;



	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param data
	 * @param columnHeaders
	 * @param columnHeaderTooltips
	 * @param isStatistics
	 */
	public JEPETTOTable(String[][] data, String[] columnHeaders, String[] columnHeaderTooltips, boolean isStatistics, Double significanceThreshold) {
		this.tooltips = columnHeaderTooltips;
		this.isStatistics = isStatistics;
		this.significanceThreshold = significanceThreshold;
		this.resultTable = createTable(data, columnHeaders);
		resultTable.setAutoCreateRowSorter(true);
	}



	/*******************
	 *	 METHODS	 *
	 *******************/

	/**
	 * Give a reference to the table
	 * which has been created
	 * @return final table
	 */
	protected JTable getCreatedTable() {
		return resultTable;
	}

	/**
	 * Creates the table
	 * @param data
	 * @param columnHeaders
	 * @return a table
	 */
	private JTable createTable(String[][] data, String[] columnHeaders) {
		return new JTable(data, columnHeaders) {
			private static final long serialVersionUID = -3007420282651595807L;

			// disable editing
			@Override
			public boolean isCellEditable (int rowIndex, int colIndex) {
				return false;
			}

			// provides custom rendering for numbers and significant rows
			@Override
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
				Component c = super.prepareRenderer(renderer, row, column);

				// read cell value
				int modelRow = convertRowIndexToModel(row);
				int modelColumn = convertColumnIndexToModel(column);
				String text = getModel().getValueAt(modelRow, modelColumn).toString();

				// if value is a number, use fixed width font
				if (hasNumbers(text)) {
					Font font = new Font(Font.MONOSPACED, Font.PLAIN, 12);
					c.setFont(font);
				}

				// highlight significant rows
				if (significanceThreshold != null && !isRowSelected(row)) {
					text = getModel().getValueAt(modelRow, 1).toString();
					try {
						Double value = Double.parseDouble(text);
						if (value >= significanceThreshold) {
							Color light_blue = new Color(186, 207, 226);
							c.setBackground(light_blue);
						} else {
							c.setBackground(getBackground());
						}
					} catch (java.lang.NumberFormatException e) {
						// do nothing if text is not a number
					}
				}
				return c;
			}

			// test if string start with numbers
			// (numbers might be negative or right aligned with space)
			private boolean hasNumbers(String text) {
				if (text.matches("-?[ ]*[0-9].*")) {
					return true;
				}
				return false;
			}

			// Implement the tooltip
			@Override
			public String getToolTipText(MouseEvent e) {
				java.awt.Point p = e.getPoint();
				int rowIndex = rowAtPoint(p);
				int colIndex = columnAtPoint(p);

				String tip = null;
				if (isStatistics) {
					int model = convertColumnIndexToModel(colIndex);

					// TODO: why these are hardcoded here instead being read
					//       like all the other tooltips?

					if (model == 0) { // names column
						if (rowIndex == 0) {
							tip = "Uploaded gene set";
						} else if (rowIndex == 1) {
							tip = "100 random simulations (mean)";
						} else if (rowIndex == 2) {
							tip = "Static mean over entire network";
						}
					} else { // other column
						if ((rowIndex == 0)|(rowIndex == 1)|(rowIndex == 2)){
							tip = (String) getValueAt(rowIndex,colIndex);
						}
					}
				} else {
					tip = (String) getValueAt(rowIndex,colIndex);
				}
				return tip;
			}

			//Implement table header tool tips
			@Override
			protected JTableHeader createDefaultTableHeader() {
				return new JTableHeader(columnModel) {

					private static final long serialVersionUID = 7035855403638521516L;

					@Override
					public String getToolTipText(MouseEvent e) {
						Point p = e.getPoint();
						int columnIndex = columnModel.getColumnIndexAtX(p.x);
						int modelIndex = columnModel.getColumn(columnIndex).getModelIndex();
						return tooltips[modelIndex];
					}
				};
			}
		};
	}
}
