/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskIterator;
import org.cytoscape.work.TaskMonitor;
import org.jfree.chart.ChartPanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Methods specific to TopoGSA allowing network topological properties analysis
 * @author cpw
 */
public class JEPETTOTopoGSAMethods extends JEPETTOGenericServerMethods {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	private static Logger logger = LoggerFactory.getLogger(JEPETTOTopoGSAMethods.class);

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 5758435113256250674L;

	// Server URL
	private static final String URL = "http://www.topogsa.org/";

	// Communication with TopoGSA server
	protected static String runID = null;

	// Comparative analysis
	protected static HashMap<String, ArrayList<ArrayList<String>>> mapValues = null;
	private static int numberOfEntries = 0;
	private static Pattern dbFormat;

	// TopoGSA result tables
	protected static JTable statTable = new JTable();
	protected static JTable maptableTable = new JTable();
	//protected static JTable compTable = new JTable();

	// Constructor
	private final String idFormatBoxItem;
	private final String comparativeAnalysisBoxItem;
	private final boolean isBias;
	private final String geneSet;


	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param idFormatBox
	 * @param comparativeAnalysisBox
	 * @param isBias
	 * @param genesetArea
	 */
	public JEPETTOTopoGSAMethods(String idFormatBoxItem, String comparativeAnalysisBoxItem,
		boolean isBias, String geneSet) {

		this.idFormatBoxItem = idFormatBoxItem;
		this.comparativeAnalysisBoxItem = comparativeAnalysisBoxItem;
		this.isBias = isBias;
		this.geneSet = geneSet;

		analysis(idFormatBoxItem, comparativeAnalysisBoxItem, isBias, geneSet);
	}


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Additional comparative analysis
	 * @param xAxis
	 * @param yAxis
	 * @param databaseBoxList
	 */
	protected static void analysis(final String xAxis, final String yAxis,
		final ArrayList<JCheckBox> databaseBoxList,
		final JEPETTOTopoGSAResultPanel resultTab) {

		AbstractTask analysisTask = new AbstractTask() {

			@Override
			public void run(TaskMonitor taskMonitor) throws ExperimentException, IOException {
				taskMonitor.setTitle("Topology comparison...");
				taskMonitor.setProgress(-1);
				taskMonitor.setStatusMessage("Sending request");

				// Send a request to TopoGSA
				String stringBox = "";
				int i = 0;
				while (i < databaseBoxList.size()) {
					stringBox += (i+1 == databaseBoxList.size()) ? databaseBoxList.get(i).getText() : databaseBoxList.get(i).getText()+"///";
					i++;
				}

				String parameters = null;
				parameters = getPostParameters(new String[]{"xaxis", "yaxis", "zaxis", "xcut", "ycut", "zcut", null, "directory", "inputchoice", "fsubmit", "checkbox"}, new String[]{xAxis, yAxis, "cc", "", "", "", stringBox, runID, "no", "Regenerate comparative plot and statistics", "no"}, new String[]{"Shortest Path Length///spl+Node Betweenness///bt+Degree///dg+Clustering Coefficient///cc+Eigenvector Centrality///evc", "Shortest Path Length///spl+Node Betweenness///bt+Degree///dg+Clustering Coefficient///cc+Eigenvector Centrality///evc", null, null, null, null, "KEGG///ckegg_kegg+BIOCARTA///cbiocarta_biocarta+GO BP///cgobp_gobp+INTERPRO///cinterpro_interpro+GO MF///cgomf_gomf+GO CC///cgocc_gocc", null, null, null, null});

				taskMonitor.setStatusMessage("Waiting for results");

				String htmlPage = CyActivator.communication.sendForm(
					URL + "index.php?tmpdat=result", setPostParameters(parameters));

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the plot computation."); return;}

				if (htmlPage == null) {
					throw new ExperimentException("TopoGSA comparison result page is empty.");
				}

				taskMonitor.setStatusMessage("Downloading results");

				// wait extra second for ranking to appear
				try {
					Thread.sleep(1000);
				} catch (InterruptedException exception) {}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the plot computation."); return;}

				// Plot
				Map<String[], Double[]> comparativeAnalysisData = getComparativeAnalysisData(htmlPage);

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the plot computation."); return;}

				setComparativeAnalysisData(comparativeAnalysisData);
				resultTab.compTable.setEnabled(true);
				resultTab.updateTable(getCompTable(), resultTab.comparativeAnalysisPanel);
				resultTab.updatePlot();
			}
		};
		TaskIterator tIt = new TaskIterator(analysisTask);
		CyActivator.taskManager.execute(tIt);
	}

	/**
	 * Start an analysis with TopoGSA for the current user's settings
	 * @param idFormatBoxItem
	 * @param comparativeAnalysisBoxItem
	 * @param isBias
	 * @param geneSet
	 */
	private void analysis(final String idFormatBoxItem,	final String comparativeAnalysisBoxItem,
		final boolean isBias, final String geneSet) {

		AbstractTask analysisTask = new AbstractTask() {

			@Override
			public void run(TaskMonitor taskMonitor) throws ExperimentException {
				taskMonitor.setTitle("Topology analysis...");
				taskMonitor.setProgress(-1);
				taskMonitor.setStatusMessage("Sending request");

				// Keep trace of topology experiments as a tooltip in results table
				String experimentSettingsTooltip = (!isBias) ? "<html>Interaction network: Large (small-scale experiments)<br>" : "<html>Interaction network: Small (large-scale experiments)<br>";
				experimentSettingsTooltip += "Identifier format: " + idFormatBoxItem + "<br>" +
						"Reference database: " + comparativeAnalysisBoxItem + "</html>";

				// Send a request to TopoGSA
				String parameters = (!isBias) ? getPostParameters(new String[]{"inputchoice", "identifier", "checkbox", "example", "geneset"}, new String[]{"no", idFormatBoxItem, comparativeAnalysisBoxItem, "0", geneSet}, new String[]{null, "ENSEMBL///ensembl_gene_id+HGNC SYMBOL///hgnc_symbol+ENTREZ GENE///entrezgene+UNIPROT/SWISSPROT///uniprot_swissprot+REFSEQ DNA///refseq_dna+PROTEIN ID///protein_id+Affy HC G110///affy_hc_g110+Affy HG Focus///affy_hg_focus+Affy HG U133 plus 2///affy_hg_u133_plus_2+Affy HG U133a///affy_hg_u133a+Affy HG U133a 2///affy_hg_u133a_2+Affy HG U133b///affy_hg_u133b+Affy HG U95a///affy_hg_u95a+Affy HG U95av2///affy_hg_u95av2+Affy HG U95b///affy_hg_u95b+Affy HG U95c///affy_hg_u95c+Affy HG U95d///affy_hg_u95d+Affy HG U95e///affy_hg_u95e+Affy HuEx-1_0-st-v2///affy_huex_1_0_st_v2+Affy HuGene-1_0-st-v1///affy_hugene_1_0_st_v1+Affy HuGene FL///affy_hugenefl+Affy U133 X3P///affy_u133_x3p+Agilent CGH 44b///agilent_cgh_44b+Agilent Wholegenome///agilent_wholegenome+Illumina HumanWG-6 v1///illumina_humanwg_6_v1+Illumina HumanWG-6 v2///illumina_humanwg_6_v2+Illumina HumanWG-6 v3///illumina_humanwg_6_v3+Gene Symbol (yeast)///yeast_gene_symbol+FlyBase Symbol///flybase_symbol+Gene Symbol (thaliana)///arabidopsis_gene_symbol+Gene Symbol (elegans)///caenorhabditis_gene_symbol", "No///no+KEGG///kegg+BIOCARTA///biocarta+GO biological process///gobp+INTERPRO///interpro+GO molecular function///gomf+GO cellular component///gocc+GO///go+MetaCyc///metacyc", null, null}) : getPostParameters(new String[]{"inputchoice", "identifier", "checkbox", "clarge", "example", "geneset"}, new String[]{"no", idFormatBoxItem, comparativeAnalysisBoxItem, "yes", "0", geneSet}, new String[]{null, "ENSEMBL///ensembl_gene_id+HGNC SYMBOL///hgnc_symbol+ENTREZ GENE///entrezgene+UNIPROT/SWISSPROT///uniprot_swissprot+REFSEQ DNA///refseq_dna+PROTEIN ID///protein_id+Affy HC G110///affy_hc_g110+Affy HG Focus///affy_hg_focus+Affy HG U133 plus 2///affy_hg_u133_plus_2+Affy HG U133a///affy_hg_u133a+Affy HG U133a 2///affy_hg_u133a_2+Affy HG U133b///affy_hg_u133b+Affy HG U95a///affy_hg_u95a+Affy HG U95av2///affy_hg_u95av2+Affy HG U95b///affy_hg_u95b+Affy HG U95c///affy_hg_u95c+Affy HG U95d///affy_hg_u95d+Affy HG U95e///affy_hg_u95e+Affy HuEx-1_0-st-v2///affy_huex_1_0_st_v2+Affy HuGene-1_0-st-v1///affy_hugene_1_0_st_v1+Affy HuGene FL///affy_hugenefl+Affy U133 X3P///affy_u133_x3p+Agilent CGH 44b///agilent_cgh_44b+Agilent Wholegenome///agilent_wholegenome+Illumina HumanWG-6 v1///illumina_humanwg_6_v1+Illumina HumanWG-6 v2///illumina_humanwg_6_v2+Illumina HumanWG-6 v3///illumina_humanwg_6_v3+Gene Symbol (yeast)///yeast_gene_symbol+FlyBase Symbol///flybase_symbol+Gene Symbol (thaliana)///arabidopsis_gene_symbol+Gene Symbol (elegans)///caenorhabditis_gene_symbol", "No///no+KEGG///kegg+BIOCARTA///biocarta+GO biological process///gobp+INTERPRO///interpro+GO molecular function///gomf+GO cellular component///gocc+GO///go+MetaCyc///metacyc", null, null, null});

				dbFormat = getPathwayPattern(comparativeAnalysisBoxItem);

				taskMonitor.setStatusMessage("Waiting for results (may take up to 1 minute)");

				String html = CyActivator.communication.sendForm(
						URL + "index.php?tmpdat=result", setPostParameters(parameters));

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the topology analysis."); return;}

				if (html == null) {
					throw new ExperimentException("TopoGSA result page is empty.");
				}

				logger.info("Form response length: {}", html.length());

				if (html.matches(".+?Unfortunately,\\snone\\sof\\sthe\\ssubmitted\\sgene\\snames\\scould\\sbe\\smapped\\sto\\sENSEMBL\\sidentifiers.+?")) {
					String text = "Unfortunately, none of your gene names could be mapped onto the background interaction network.";
					JOptionPane.showMessageDialog(null, text);
					return;
				}

				// store current experiment ID
				String previousID = runID;

				// find experiment ID
				runID = getID(html, "([a-z0-9]{5}_[0-9]{10})");
				if (runID == null) {
					throw new ExperimentException("TopoGSA experiment ID not found.");
				} else {
					logger.info("Experiment ID: {}", runID);
				}

				logger.info("Reading topological properties.");

				// Comparative analysis
				ArrayList<Object> plottingData = plot(html);

				taskMonitor.setStatusMessage("Downloading results");

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the topology analysis."); runID = previousID; return;}

				logger.info("Reading network statistics.");

				JTable mainStatisticsResult = null;	// Statistics
				JTable mappingInformationResult = null;	// Mapping data

				mainStatisticsResult = getTableFromFile(URL+"pages/tmp/"+runID+"/stat.html", "readDataFromStat", new String[]{"","SPL","NB","D","CC","EC"}, new String[]{"", "Shortest path length", "Node betweenness", "Degree", "Clustering coefficient", "Eigenvector centrality"}, true, 4);

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the topology analysis."); runID = previousID; return;}

				// wait extra 2 seconds for mapping table to appear
				try {
					Thread.sleep(2000);
				} catch (InterruptedException exception) {}

				// check if the task has been cancelled
				if (this.cancelled) { logger.info("User stopped the topology analysis."); runID = previousID; return;}

				logger.info("Reading mapping data file.");

				mappingInformationResult = getTableFromFile(URL+"pages/tmp/"+runID+"/maptable.html", "readDataFromMaptable", new String[]{"Initial","Converted","Presence"}, new String[]{"Initial identifier", "Converted identifier", "Present in network"}, false, 4);

				CyActivator.overallTopologyAnalysisNumber++;

				// Update & select the results panel
				JEPETTOTopoGSAResultPanel basicResultPanel = new JEPETTOTopoGSAResultPanel(experimentSettingsTooltip);
				basicResultPanel.updateTable(mainStatisticsResult, basicResultPanel.mainStatisticsPanel);
				basicResultPanel.updateTable(mappingInformationResult, basicResultPanel.mappingPanel);
				basicResultPanel.updateMappingLabelToMappedGenesNb(mappingInformationResult);
				basicResultPanel.compTable.setEnabled(true);
				basicResultPanel.updateTable((JTable) plottingData.get(0), basicResultPanel.comparativeAnalysisPanel);
				basicResultPanel.saveComp.setEnabled(true);
				basicResultPanel.updatePlot();

				CyActivator.topologyResultsTab.getPanel();
			}

		};

		TaskIterator it = new TaskIterator(analysisTask);
		CyActivator.taskManager.execute(it);
	}

	/**
	 * Plot the comparative analysis
	 * @param htmlPage
	 */
	private static ArrayList<Object> plot(String htmlPage) {
		Map<String[], Double[]> data;
		ArrayList<Object> plotData = new ArrayList<Object>();

		// FIXME: think about this, is error handling here really needed?
		try {
			 data = getComparativeAnalysisData(htmlPage);
		} catch (ExperimentException e) {
			logger.error("Comparative analysis data not read: {}", e.getMessage());
			return plotData;
		}

		setComparativeAnalysisData(data);
		plotData.add(getCompTable());
		plotData.add(plotValues("Shortest Path Length", "Node Betweenness"));
		return plotData;
	}

	/**
	 * Set the comparative analysis data
	 * in a specific format
	 * @param data
	 */
	private static void setComparativeAnalysisData(Map<String[], Double[]> data) {
		numberOfEntries = 0;
		mapValues = new HashMap<String,ArrayList<ArrayList<String>>>();

		for (Map.Entry<String[], Double[]> entry : data.entrySet()) {
			Double[] values = entry.getValue();
			String[] keys = entry.getKey();

			ArrayList<String> row = new ArrayList<String>();
			row.add(keys[0]);

			for (int i = 0; i < 7; ++i) {
				String format = "%4.2f";  // CC, EC, distance
				if (i == 0) {
					format = "%4.1f";  // degree
				} else if (i == 2) {
					format = "%5.2f";  // SPL
				} else if (i == 3) {
					format = "%6.0f";  // BC
				} else if (i == 5) {
					format = "%.0f";  // number of genes
				}
				row.add(String.format(format, values[i]));
			}

			boolean hasKey = mapValues.containsKey(keys[1]);
			if (!hasKey) {
				mapValues.put(keys[1], new ArrayList<ArrayList<String>>());
			}
			mapValues.get(keys[1]).add(row);
			++numberOfEntries;
		}
	}

	/**
	 * Get the comparative analysis data from the TopoGSA
	 * @param sourcePage
	 * @return comparative analysis data
	 */
	private static Map<String[], Double[]> getComparativeAnalysisData(String sourcePage)
	throws ExperimentException {
		Map<Integer, String> plotLabels = new JEPETTOParser(sourcePage).readLabelFromSvgplot();

		// read input gene set signature
		String top5Table = CyActivator.communication.readFile(URL+"pages/tmp/"+runID+"/top5.html");
		Map<String[],Double[]> top5Data = new JEPETTOParser(top5Table).readDataFromTop5();

		// read pathway ranking (signatures and distance)
		String rankingTable = CyActivator.communication.readFile(URL+"pages/tmp/"+runID+"/rankingtop.html");
		Double[] rankingtopData = new JEPETTOParser(rankingTable).readDataFromRankingtop();

		Map<String[], Double[]> mergedData = JEPETTOParser.mergeSvgplotTop5(top5Data, plotLabels);
		String[] s = {"Uploaded gene set", "Uploaded gene set (median)"};
		mergedData.put(s, rankingtopData);
		return mergedData;
	}

	/**
	 * Generate the plot of the comparative analysis
	 * @param x
	 * @param y
	 * @return a panel containing the plot
	 */
	private static ChartPanel plotValues(String x, String y) {
		JEPETTOGenerateScatterPlot plot = new JEPETTOGenerateScatterPlot(mapValues, x, y);
		return plot.getChartPanel();
	}

	/**
	 * Generate table with comparative analysis data
	 */
	private static JTable getCompTable() {
		String[][] values = new String[numberOfEntries][8];

		int row = 0;
		for (Map.Entry<String, ArrayList<ArrayList<String>>> l : mapValues.entrySet()) {
			for (ArrayList<String> o : l.getValue()) {
				if (row == 0) {
					values[row][0] = o.get(0);
				} else {
					Matcher m = dbFormat.matcher(o.get(0));
					if (m.find()) {
						values[row][0] = m.group(2) + " - " + m.group(1);
					}
				}

				for (int col = 2; col < 7; ++col) {
					values[row][col] = o.get(col-1);
				}

				values[row][0] += " (" + o.get(6) + ")";
				values[row][1] = l.getKey();
				values[row][7] = o.get(7);
				row++;
			}
		}
		JTable table = getTable(values, new String[]{"Identifier", "Class", "D", "CC", "SPL", "NB", "EC", "Score"}, new String[]{"Identifier", "Class", "Node Degree", "Clustering Coefficient", "Shortest Path Length", "Node betweenness",	"Eigenvector Centrality", "Score"}, false, true, 4, Double.NEGATIVE_INFINITY);
		return table;
	}

	/**
	 * Get database pathway annotation pattern to display only name
	 * @param db
	 * @return annotation database pattern
	 */
	private Pattern getPathwayPattern(String db) {
		Pattern p = Pattern.compile("^(.+?):(.+)$");
		if (db.matches("KEGG") | db.matches("BIOCARTA") | db.matches("INTERPRO")) {
			p = Pattern.compile("^(.+?):(.+)$");
		} else if (db.matches("GO biological process") | db.matches("GO molecular function") | db.matches("GO cellular component")) {
			p = Pattern.compile("^(GO:[0-9]{7})~(.+)$");
		}
		return p;
	}



}
