/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.commons.lang.StringUtils;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.model.CyTableUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JEPETTOTopoGSAPanel extends JPanel {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	private final Logger logger = LoggerFactory.getLogger(getClass());

	// Serial Version UID (auto-generated)
	private static final long serialVersionUID = -7367866909580254169L;

	// UI components
	private static JComboBox idFormatBox = null;
	private static JComboBox annotationDbBox = null;
	private static JComboBox interactionNetworkBox = null;

	private static boolean useSmallNetwork = false;

	private static JTextArea idArea = null;

	private static JButton inputSetStartButton = new JButton("Use gene list");
	protected static JButton selectedNodesStartButton = new JButton("Use selected nodes");


	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param mainPanel
	 */
	public JEPETTOTopoGSAPanel() {
		super();
		setGeneralLayout();
	}

	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Create the general layout of the GUI
	 */
	private void setGeneralLayout() {
		this.setLayout(new BorderLayout(2,6));

		JPanel northPanel = setNorthPanel();
		JPanel centerPanel = setCenterPanel();
		JPanel southPanel = setSouthPanel();

		this.add(northPanel, BorderLayout.NORTH);
		this.add(centerPanel, BorderLayout.CENTER);
		this.add(southPanel, BorderLayout.SOUTH);
	}

	/**
	 * Create the northern panel of the GUI
	 * Corresponds to id formats, comparative analysis
	 * and bias reduction parameters
	 * @param panel: northern panel
	 */
	private JPanel setNorthPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(3,2,2,4));

		// First row
		JLabel isBiasLabel = new JLabel("<html><b>Interaction network:</b></html>");
		panel.add(isBiasLabel);	// west

		String[] items = {"Large (small-scale experiments)", "Small (large-scale experiments)"};
		interactionNetworkBox = new JComboBox(items);

		String text =  "<html>The default interaction network used in topological analysis"
				+ "<br />has 9392 nodes and 38857 edges and includes interactions derived"
				+ "<br />from <b>small-scale experiments</b>.<br /><br />"
				+ "To avoid potential bias a smaller network (2750 nodes and 5621 interactions)"
				+ "<br />derived from <b>genome-scale experiments</b> could be used"
				+ "<br />if the gene list contains only human data.</html>";
		interactionNetworkBox.setToolTipText(text);

		interactionNetworkBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (interactionNetworkBox.getSelectedItem().toString().startsWith("S")) {
					useSmallNetwork = true;
				} else {
					useSmallNetwork = false;
				}
			}
		});
		panel.add(interactionNetworkBox);	// east

		// Second row
		JLabel idFormatLabel = new JLabel("<html><b>Identifier format:</b></html>");
		panel.add(idFormatLabel);	// west
		setupIdFormatBox();
		panel.add(idFormatBox);	// east

		// Third row
		JLabel comparativeAnalysisLabel = new JLabel("<html><b>Reference database:</b></html>");
		panel.add(comparativeAnalysisLabel);	// west
		setupAnnotationDbBox();
		panel.add(annotationDbBox);	// east

		return panel;
	}

	/**
	 * Create the central panel of the GUI
	 * Corresponds to the gene set parameter
	 * @param panel: central panel
	 */
	private JPanel setCenterPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout(2,4));

		// Gene set area label
		JLabel geneSetLabel = new JLabel("<html><b>Gene/protein set</b> (1 per line):</html>");
		panel.add(geneSetLabel, BorderLayout.NORTH);

		// Gene set area
		idArea = new JTextArea();
		idArea.setToolTipText("Add gene/protein names to the list or select nodes in the current network.");

		idArea.getDocument().addDocumentListener(new DocumentListener() {
	        @Override
	    	public void removeUpdate(DocumentEvent e) {
				if (idArea.getText().isEmpty()) {
					inputSetStartButton.setEnabled(false);
				}
			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				inputSetStartButton.setEnabled(true);
			}

			@Override
			public void changedUpdate(DocumentEvent arg0) {
				if (idArea.getText().isEmpty()) {
					inputSetStartButton.setEnabled(false);
				} else {
					inputSetStartButton.setEnabled(true);
				}
			}
		});

		JScrollPane geneSetScroll = new JScrollPane(idArea);
		panel.add(geneSetScroll, BorderLayout.CENTER);

		return panel;
	}

	/**
	 * Create the southern panel of the GUI
	 * Corresponds to the start button
	 * @param panel: southern panel
	 */
	private JPanel setSouthPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(1, 2));

		selectedNodesStartButton = new JButton("Use selected nodes");
		selectedNodesStartButton.setToolTipText("Start the analysis using selected nodes in the current network.");
		selectedNodesStartButton.setEnabled(false);

		selectedNodesStartButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				LinkedList<String> labels = getLabelsOfSelectedNodes();
				if (labels == null) {
					return;
				}

				idArea.setText(StringUtils.join(labels, System.getProperty("line.separator")));
				idArea.repaint();

				if (isInputSizeOK(labels.size())) {
					String inputSet = StringUtils.join(labels, "\r\n");
					runExperiment(inputSet);
				}
			}
		});

		inputSetStartButton = new JButton("Use gene list");
		inputSetStartButton.setToolTipText("Start the analysis using names on the list.");
		inputSetStartButton.setEnabled(false);

		inputSetStartButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				String text = idArea.getText();
				String[] lines = text.split("\\r?\\n");

				if (isInputSizeOK(lines.length)) {
					String inputSet = StringUtils.join(lines, "\r\n");
					runExperiment(inputSet);
				}
			}
		});

		panel.add(inputSetStartButton);
		panel.add(selectedNodesStartButton);

		return panel;
	}

	/**
	 * Check whether the submitted input set size is in range [10,2000].
	 * @param inputSet
	 * @return true if size within limits, otherwise false
	 */
	private boolean isInputSizeOK(int size) {
		if (size > 2000) {
			JOptionPane.showMessageDialog(null, "Please reduce the input set size to 2000 genes (currently "
				+ Integer.toString(size) + ").", "Size limit reached!", JOptionPane.WARNING_MESSAGE);
			return false;
		}
		if (size < 10) {
			JOptionPane.showMessageDialog(null, "Please input at least 10 genes.", "Input set too small!", JOptionPane.WARNING_MESSAGE);
			return false;
		}
		return true;
	}

	private void runExperiment(String inputSet) {
		new JEPETTOTopoGSAMethods((String) idFormatBox.getSelectedItem(),
			(String) annotationDbBox.getSelectedItem(), useSmallNetwork, inputSet);
	}

	/**
	 * Reads labels of the currently selected network nodes.
	 */
	protected LinkedList<String> getLabelsOfSelectedNodes() {
		CyNetwork currentNetwork = CyActivator.cyApplicationManagerService.getCurrentNetwork();
		if (currentNetwork == null) {
			JOptionPane.showMessageDialog(null, "Network not found. Please load a network and select nodes first.");
			return null;
		}

		List<CyNode> nodes = CyTableUtil.getNodesInState(currentNetwork, "selected", true);
		if (nodes.size() == 0) {
			JOptionPane.showMessageDialog(null, "There are no nodes selected nodes in the current network view.");
			return null;
		}

		LinkedList<String> labels = new LinkedList<String>();
		for (CyNode node : nodes) {
			String label = currentNetwork.getRow(node).get(CyNetwork.NAME, String.class);
			labels.add(label);
		}

		return labels;
	}

	/**
	 * Import target set from file
	 * (1 per line)
	 */
	protected String importFromFile() {
		String targetList = "";
		JFileChooser chooser = new JFileChooser();
		int resp = chooser.showOpenDialog(chooser);
		if (resp == JFileChooser.APPROVE_OPTION) {
			try {
				BufferedReader br = new BufferedReader(new FileReader(chooser.getSelectedFile().getAbsolutePath()));
				String line;
				while ((line = br.readLine()) != null) {
					if (line.split("\\s").length > 1) {
						targetList += line.split("\\s")[0] + System.getProperty("line.separator");
					} else {
						targetList += line + System.getProperty("line.separator");
					}
				}
				br.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if (targetList.equals("")) {
			JOptionPane.showMessageDialog(null, "The file you tried to import does not contain any ID.");
		}
		return targetList;
	}

	/**
	 * Set identifiers format box content
	 */
	private static void setupIdFormatBox() {
		String[] items = {"HGNC SYMBOL", "ENSEMBL", "ENTREZ GENE",
			"UNIPROT/SWISSPROT", "REFSEQ DNA", "PROTEIN ID", "Affy HC G110",
			"Affy HG Focus", "Affy HG U133 plus 2", "Affy HG U133a",
			"Affy HG U133a 2", "Affy HG U133b", "Affy HG U95a",
			"Affy HG U95av2", "Affy HG U95b", "Affy HG U95c", "Affy HG U95d",
			"Affy HG U95e", "Affy HuEx-1_0-st-v2", "Affy HuGene-1_0-st-v1",
			"Affy HuGene FL", "Affy U133 X3P", "Agilent CGH 44b",
			"Agilent Wholegenome", "Illumina HumanWG-6 v1",
			"Illumina HumanWG-6 v2", "Illumina HumanWG-6 v3"};

		idFormatBox = new JComboBox(items);
		idFormatBox.setToolTipText("Choose a naming convention used for the input set.");
		idFormatBox.setSelectedIndex(0);
	}

	/**
	 * Set annotation database box content
	 */
	private static void setupAnnotationDbBox() {
		String[] items = {"KEGG", "BIOCARTA", "INTERPRO", "GO biological process",
			"GO molecular function", "GO cellular component"};

		annotationDbBox = new JComboBox(items);
		annotationDbBox.setToolTipText("Choose a reference database for topology comparison.");
		annotationDbBox.setSelectedIndex(0);
	}

}
