/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import org.jfree.chart.ChartPanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JEPETTOTopoGSAResultPanel extends JPanel {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	private final Logger logger = LoggerFactory.getLogger(getClass());

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 4263286782575604636L;

	// Constructor panel
	private final JTabbedPane resultsPanel;

	// UI updated components
	protected JPanel mainStatisticsPanel = new JPanel();
	protected JPanel comparativeAnalysisPanel = new JPanel();
	protected JPanel mappingPanel = new JPanel();
	private final JLabel mappingLabel = new JLabel();
	protected JPanel scatterPlotPanel = new JPanel();

	// Comparative analysis components
	private static String[] plotAxis = {
		"Shortest Path Length",
		"Node Betweenness",
		"Degree",
		"Clustering Coefficient",
		"Eigenvector Centrality",
		"Number of Genes",
		"Score"};
	protected JComboBox xBox = new JComboBox(plotAxis);
	protected JComboBox yBox = new JComboBox(plotAxis);
	protected JButton saveComp = new JButton("Save");

	// Reference datasets
	protected JCheckBox keggDbBox = new JCheckBox("KEGG");
	protected JCheckBox biocartaDbBox = new JCheckBox("BIOCARTA");
	protected JCheckBox gobpDbBox = new JCheckBox("GO BP");
	protected JCheckBox interproDbBox = new JCheckBox("INTERPRO");
	protected JCheckBox gomfDbBox = new JCheckBox("GO MF");
	protected JCheckBox goccDbBox = new JCheckBox("GO CC");
	private final JCheckBox[] databaseBoxList = {keggDbBox, biocartaDbBox,
			gobpDbBox, interproDbBox, gomfDbBox, goccDbBox};

	// New analysis
	private final JButton newAnalysisButton = new JButton("Generate plot");
	protected JRadioButton compTable = new JRadioButton("Comparison");
	protected String tableToSave = "Statistics";

	// Database memory
	private boolean[] databaseSelectedMemory = {true, false, false, false, false, false};

	// Keep reference to current panel
	private final JEPETTOTopoGSAResultPanel currentPanel;




	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 */
	public JEPETTOTopoGSAResultPanel(String experimentTraceTooltip) {
		currentPanel = this;
		JPanel main = new JPanel(new BorderLayout());
		CyActivator.topologyResultsTab.getResultsTabbedPanel().add(CyActivator.overallTopologyAnalysisNumber+". Topology", main);
		JPanel pnlTab = new JPanel(new GridBagLayout());
		pnlTab.setOpaque(false);
		JLabel lblTitle = new JLabel(CyActivator.overallTopologyAnalysisNumber+". Topology");
		lblTitle.setToolTipText(experimentTraceTooltip);
		JButton btnClose = new JButton("x");
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		pnlTab.add(lblTitle, gbc);
		gbc.gridx++;
		gbc.weightx = 0;
		pnlTab.add(btnClose, gbc);
		CyActivator.topologyResultsTab.getResultsTabbedPanel().setTabComponentAt(CyActivator.topologyResultsTab.getResultsTabbedPanel().getTabCount()-1, pnlTab);
		btnClose.addActionListener(new JEPETTOCloseTabHandling(CyActivator.overallTopologyAnalysisNumber+". Topology", CyActivator.topologyResultsTab));
		resultsPanel = new JTabbedPane();
		resultsPanel.setTabPlacement(JTabbedPane.LEFT);
		main.add(resultsPanel, BorderLayout.CENTER);
		displayGeneralLayout();
		CyActivator.topologyResultsTab.getResultsTabbedPanel().setSelectedIndex(CyActivator.topologyResultsTab.getResultsTabbedPanel().getTabCount()-1);
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Update mapping table title to
	 * the number of mapped genes in the
	 * background interaction network
	 * @param mappingTable: table of input gene set mapping values on the background interaction network
	 */
	protected void updateMappingLabelToMappedGenesNb(JTable mappingTable) {
		int presenceIndex = mappingTable.getColumn("Presence").getModelIndex();
		int realIndex = mappingTable.convertColumnIndexToModel(presenceIndex);
		int mappedGenesNumber = 0;
		for (int row = 0 ; row < mappingTable.getRowCount() ; row++)
			if (mappingTable.getValueAt(row, realIndex).equals("yes")) mappedGenesNumber++;
		mappingLabel.setText("<html><b>" + mappedGenesNumber + " out of " + mappingTable.getRowCount() + " genes mapped onto the interaction network:</b></html>");
		mappingPanel.validate();
		mappingPanel.repaint();
	}

	/**
	 * General structure
	 */
	private void displayGeneralLayout() {
		// Statistics panel
		JPanel statisticsPanel = new JPanel();
		JScrollPane statisticsScoll = new JScrollPane(statisticsPanel);
		resultsPanel.add("Statistics", statisticsScoll);
		displayStatisticsPanel(statisticsPanel);
		// Mapping panel
		JPanel mappingDataPanel = new JPanel();
		JScrollPane mappingScroll = new JScrollPane(mappingDataPanel);
		resultsPanel.add("Mapping", mappingScroll);
		displayMappingPanel(mappingDataPanel);
		// Plot panel
		JPanel plotPanel = new JPanel();
		JScrollPane plotScroll = new JScrollPane(plotPanel);
		resultsPanel.add("Comparison", plotScroll);
		displayPlotPanel(plotPanel);
	}

	/**
	 * Statistics tab
	 * @param panel
	 */
	private void displayStatisticsPanel(JPanel panel) {
		int height = (int)(0.5 * Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		panel.setPreferredSize(new Dimension(600, height));

		panel.setLayout(new BorderLayout(2,4));

		// Main statistics panel
		JPanel northPanel = new JPanel();
		panel.add(northPanel, BorderLayout.NORTH);
		northPanel.setLayout(new BorderLayout());
		JPanel statPan = new JPanel(new BorderLayout());
		JLabel mainStatisticsLabel = new JLabel("<html><b>Gene/Protein set network topological properties:</b></html>");
		statPan.add(mainStatisticsLabel, BorderLayout.CENTER);
		JButton saveStat = new JButton("Save");
		statPan.add(saveStat, BorderLayout.EAST);
		saveStat.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new JEPETTOSaveTable("Statistics", mainStatisticsPanel, null);
			}
		});
		northPanel.add(statPan, BorderLayout.NORTH);
		mainStatisticsPanel = new JPanel();
		northPanel.add(mainStatisticsPanel, BorderLayout.CENTER);	// updated component

		// Comparative analysis data panel
		JPanel centerPanel = new JPanel();
		panel.add(centerPanel, BorderLayout.CENTER);
		centerPanel.setLayout(new BorderLayout());
		JPanel compPan = new JPanel(new BorderLayout());
		JLabel comparativeAnalysisLabel = new JLabel("<html><b>Comparison to reference database:</b></html>");
		compPan.add(comparativeAnalysisLabel, BorderLayout.CENTER);
		saveComp.setEnabled(false);
		compPan.add(saveComp, BorderLayout.EAST);
		saveComp.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new JEPETTOSaveTable("Comparison", comparativeAnalysisPanel, null);
			}
		});
		centerPanel.add(compPan, BorderLayout.NORTH);
		comparativeAnalysisPanel = new JPanel();
		centerPanel.add(comparativeAnalysisPanel, BorderLayout.CENTER);	// updated component
		JButton fastComparativeAnalysis = new JButton("Generate comparative analysis");
		fastComparativeAnalysis.setToolTipText("Compares topological values of the target set with pathways/processes in the reference database.");
		comparativeAnalysisPanel.add(fastComparativeAnalysis);

		fastComparativeAnalysis.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {

				// FIXME: why this special case is needed?
				if (xBox.getSelectedItem().equals("Score") | xBox.getSelectedItem().equals("Number of Genes") | yBox.getSelectedItem().equals("Score") | yBox.getSelectedItem().equals("Number of Genes")) {
					runExperiment("Shortest Path Length", "Node Betweenness");
				} else {
					runExperiment((String) xBox.getSelectedItem(), (String) yBox.getSelectedItem());
				}
				saveComp.setEnabled(true);
			}
		});
	}


	private void runExperiment(String x, String y) {
		ArrayList<JCheckBox> selectedDatabases = new ArrayList<JCheckBox>();
		for (JCheckBox box : databaseBoxList) {
			if (box.isSelected()) {
				selectedDatabases.add(box);
			}
		}

		JEPETTOTopoGSAMethods.analysis(x, y, selectedDatabases, this);
	}


	/**
	 * Mapping tab
	 * @param panel
	 */
	private void displayMappingPanel(JPanel panel) {
		int height = (int)(0.5 * Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		panel.setPreferredSize(new Dimension(600, height));

		panel.setLayout(new BorderLayout(2,4));

		// Mapping data
		JPanel mapPanel = new JPanel(new BorderLayout());
		mapPanel.add(mappingLabel, BorderLayout.CENTER);
		JButton saveMapping = new JButton("Save");
		mapPanel.add(saveMapping, BorderLayout.EAST);
		saveMapping.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new JEPETTOSaveTable("Mapping", mappingPanel, null);
			}
		});
		panel.add(mapPanel, BorderLayout.NORTH);
		panel.add(mappingPanel, BorderLayout.CENTER);	// updated component
	}

	/**
	 * Plots tab
	 * @param panel
	 */
	private void displayPlotPanel(JPanel panel) {
		int height = (int)(0.5 * Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		panel.setPreferredSize(new Dimension(600, height));

		//TODO: change to BoxLayout (as in regression plot tab)
		panel.setLayout(new BorderLayout(2,4));

		// Comparative analysis parameters
		JPanel parametersPanel = new JPanel();
		panel.add(parametersPanel, BorderLayout.NORTH);
		parametersPanel.setLayout(new GridLayout(6,2,2,4));

		JLabel xAxisLabel = new JLabel("<html><b>Please choose the X axis:</b></html>");
		parametersPanel.add(xAxisLabel);
		xBox = new JComboBox(plotAxis);
		xBox.setSelectedIndex(0);
		parametersPanel.add(xBox);

		JLabel yAxisLabel = new JLabel("<html><b>Please choose the Y axis:</b></html>");
		parametersPanel.add(yAxisLabel);
		yBox = new JComboBox(plotAxis);
		yBox.setSelectedIndex(2);
		parametersPanel.add(yBox);

		JLabel additionalAnalysisLabel = new JLabel("<html><b>New comparative analysis?</b></html>");
		parametersPanel.add(additionalAnalysisLabel);

		JLabel maxDatabaseLabel = new JLabel("(max. 2 datasets)");
		parametersPanel.add(maxDatabaseLabel);

		addDatabasePanel("<html><em>Pathways and Processes:</em></html>",
				parametersPanel,
				new JCheckBox[]{keggDbBox, biocartaDbBox, gobpDbBox});
		keggDbBox.setSelected(true);

		addDatabasePanel("<html><em>Sequence-based functions:</em></html>",
				parametersPanel,
				new JCheckBox[]{interproDbBox, gomfDbBox});
		addDatabasePanel("<html><em>Localization:</em></html>",
				parametersPanel,
				new JCheckBox[]{goccDbBox});

		// Plot
		scatterPlotPanel = new JPanel();
		panel.add(scatterPlotPanel, BorderLayout.CENTER);	// updated component

		// Generate new plot
		JPanel newAnalysisPanel = new JPanel();
		panel.add(newAnalysisPanel, BorderLayout.SOUTH);

		JPanel testSouthPanel = new JPanel(new BorderLayout());
		newAnalysisPanel.setLayout(new BorderLayout());
		newAnalysisPanel.add(testSouthPanel, BorderLayout.NORTH);

		testSouthPanel.setLayout(new GridLayout(1,2,2,4));
		JLabel newAnalysisLabel = new JLabel("<html><em>Plot with new parameters:</em></html>");
		testSouthPanel.add(newAnalysisLabel);
		testSouthPanel.add(newAnalysisButton);

		newAnalysisButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				if (checkMaxDatabase() == 0) {
					updatePlot();
				} else {
					ArrayList<JCheckBox> databaseCheckedBoxList = new ArrayList<JCheckBox>();
					int i = 0;
					boolean[] testMemory = new boolean[6];
					for (JCheckBox box : databaseBoxList) {
						testMemory[i] = checkIfSelected(box);
						if (box.isSelected()) {
							databaseCheckedBoxList.add(box);
						}
						i++;
					}
					if ((compareDatabaseMemory(testMemory, currentPanel.databaseSelectedMemory))) {
						updatePlot();
					} else {
						if (xBox.getSelectedItem().equals("Score") | xBox.getSelectedItem().equals("Number of Genes") | yBox.getSelectedItem().equals("Score") | yBox.getSelectedItem().equals("Number of Genes")) {
							runExperiment("Shortest Path Length", "Node Betweenness");
						} else {
							runExperiment((String) xBox.getSelectedItem(),
								(String) yBox.getSelectedItem());
						}

						databaseSelectedMemory = testMemory;
						saveComp.setEnabled(true);
					}
				}
			}
		});
		JLabel saveLabel = new JLabel("To save a graph, right click on it and choose the \"Save as...\" option.");
		newAnalysisPanel.add(saveLabel, BorderLayout.CENTER);
	}

	/**
	 * Update any table with new values
	 * @param table
	 * @param panel
	 */
	protected void updateTable(JTable table, JPanel panel) {
		JScrollPane scroll = null;
		panel.removeAll();
		panel.setLayout(new BorderLayout());
		if (panel == mainStatisticsPanel) {
			panel.add(table.getTableHeader(), BorderLayout.PAGE_START);
			panel.add(table, BorderLayout.CENTER);
		} else {
			scroll = new JScrollPane(table);
			panel.add(table.getTableHeader(), BorderLayout.PAGE_START);
			panel.add(scroll, BorderLayout.CENTER);
		}
		panel.validate();
	}

	/**
	 * Update the comparative analysis plot with the current data
	 */
	protected void updatePlot() {
		JEPETTOGenerateScatterPlot plot = new JEPETTOGenerateScatterPlot(
			JEPETTOTopoGSAMethods.mapValues, (String) xBox.getSelectedItem(),
			(String) yBox.getSelectedItem());

		ChartPanel chart = plot.getChartPanel();

		//TODO: resize to the parent panel width
		int height = (int)(0.33 * Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		chart.setPreferredSize(new Dimension(600, height));

		scatterPlotPanel.removeAll();
		scatterPlotPanel.add(chart);
		scatterPlotPanel.validate();
	}

	/**
	 * Add reference databases to the comparative analysis panel
	 * @param databaseLabel
	 * @param panel
	 * @param boxList
	 */
	private void addDatabasePanel(String databaseLabel, JPanel panel, JCheckBox[] boxList) {
		JLabel label = new JLabel(databaseLabel);
		panel.add(label);
		JPanel boxPanel = new JPanel();
		panel.add(boxPanel);
		boxPanel.setLayout(new GridLayout(1,3));
		addDatabaseCheckBoxes(boxList, boxPanel);
	}

	/**
	 * Add database checkboxes with the global properties
	 * @param boxList
	 * @param panel
	 */
	private void addDatabaseCheckBoxes(JCheckBox[] boxList, JPanel panel) {
		for (final JCheckBox box : boxList) {
			panel.add(box);
			box.addActionListener(new ActionListener(){
				@Override
				public void actionPerformed(ActionEvent e){
					if (checkMaxDatabase() == 0) {
						newAnalysisButton.setEnabled(false);
					} else {
						newAnalysisButton.setEnabled(true);
					}
					if (checkMaxDatabase() > 2) {
						box.setSelected(false);
					}
				}
			});
			box.setSelected(false);
		}
	}

	/**
	 * Compare if the database currently used is the same than in the previous experiment
	 * @param a1
	 * @param a2
	 * @return if they are identical
	 */
	private boolean compareDatabaseMemory(boolean[] a1, boolean[] a2) {
		int i = 0;
		while (i < a1.length) {
			if (a1[i] != a2[i]) {
				return false;
			}
			i++;
		}
		return true;
	}

	/**
	 * check if a database is selected
	 * @param box
	 * @return the selected statement
	 */
	private boolean checkIfSelected(JCheckBox box) {
		return (box.isSelected()) ? true : false ;
	}

	/**
	 * Check that there is no more than 2 selected databases
	 * @return the number of selected databases
	 */
	private int checkMaxDatabase() {
		int counter = 0;
		for (JCheckBox box : databaseBoxList) {
			counter += (box.isSelected()) ? 1 : 0;
		}
		return counter;
	}
}
